
fit2df <- function(f, param, hdi=NULL) {
  if (is.null(f)) {
    stop(paste("Fit is NULL for param", param))
  }
  tryCatch({
    df <- data.frame( param = param, distribution = f$distname, fitparam = names(f$estimate), estimate = as.numeric(f$estimate), sd = as.numeric(f$sd), stringsAsFactors = FALSE )
    if (!is.null(f$fix.arg)) {
      df <- rbind(df,
                  data.frame( param = param, distribution = f$distname, fitparam = names(f$fix.arg), estimate = as.numeric(f$fix.arg), sd = 0, stringsAsFactors = FALSE )
      )
    }
    if (!is.null(hdi)) {
      p <- paste0("hdi", attr(hdi, "Probability"))
      df <- rbind(df,
                  data.frame( param = param, distribution = p, fitparam = c("low", "high"), estimate = as.numeric(hdi), sd = NA, stringsAsFactors = FALSE )
      )
    }
  },
  error = function(e) {
    print(f)
    print(e)
    stop(paste("Cannot print fit for param", param))
  }
  )
  df
}

dstudent <- function(x, nu, mean, sigma, log = FALSE) {
  if (log) {
    dt((x-mean)/sigma, nu, log=T)-log(sigma)
  } else {
    dt((x-mean)/sigma, nu, log=F)/sigma
  }
}
pstudent <- function(q, nu, mean, sigma, lower.tail = TRUE, log.p = FALSE) pt((q-mean)/sigma, nu, lower.tail = lower.tail, log.p=log.p)
qstudent <- function(p, nu, mean, sigma) (qt(p, nu)*sigma+mean)

dlstudent <- function(x, nu, mean, sigma, log = FALSE) {
  if (log) {
    dt((log(x)-mean)/sigma, nu, log=T)-log(sigma)-log(x)
  } else {
    dt((log(x)-mean)/sigma, nu, log=F)/sigma/x
  }
}
plstudent <- function(q, nu, mean, sigma, lower.tail = TRUE, log.p = FALSE) pt((log(q)-mean)/sigma, nu, lower.tail = lower.tail, log.p=log.p)
qlstudent <- function(p, nu, mean, sigma) exp(qt(p, nu)*sigma+mean)

dshiftlognormal <- function(x, mean, sigma, shift, log = FALSE) dlnorm(x+shift, mean, sigma, log=log)
pshiftlognormal <- function(q, mean, sigma, shift, lower.tail = TRUE, log.p = FALSE) plnorm(q+shift, mean, sigma, lower.tail = lower.tail, log.p=log.p)
qshiftlognormal <- function(p, mean, sigma, shift, log.p = FALSE) qlnorm(p, mean, sigma, log.p=log.p)-shift

dshiftlstudent <- function(x, nu, mean, sigma, shift, log = FALSE) {
  if (log) {
    dt((log(x+shift)-mean)/sigma, nu, log=T)-log(sigma)-log(x+shift)
  } else {
    dt((log(x+shift)-mean)/sigma, nu, log=F)/sigma/(x+shift)
  }
}
pshiftlstudent <- function(q, nu, mean, sigma, shift, lower.tail = TRUE, log.p = FALSE) pt((log(q+shift)-mean)/sigma, nu, lower.tail = lower.tail, log.p = log.p)
qshiftlstudent <- function(p, nu, mean, sigma, shift) exp(qt(p, nu)*sigma+mean)-shift

fitStudentOrNormal <- function(y) {
  f <- NULL
  try({
    f <- suppressWarnings(fitdistrplus::fitdist(y, "student", start=list(nu=30, mean=mean(y), sigma=sd(y))))
  }, silent = TRUE)
  if (is.null(f) || f$estimate[1] > 100) {
    f <- fitdistrplus::fitdist(y, "norm")
  }
  f
}


fitShiftedLogNormal <- function(y, fallback=T) {
  f <- NULL
  D <- 1-min(y)

  try({
    f0 <- fitdistrplus::fitdist(y+D, "lnorm")
    f <- NULL
    for (om in c("BFGS", "Nelder-Mead")) {
      try(f <- fitdistrplus::fitdist(y, "shiftlognormal",
                       start=list(mean=as.numeric(f0$estimate["meanlog"]),
                                  sigma=as.numeric(f0$estimate["sdlog"]),
                                  shift=D),
                       optim.method = om), silent = T)
      if (!is.null(f)) break
    }
  },
  silent = TRUE)

  if (!fallback) return(f)

  if (is.null(f)) {
    f1 <- NULL
    f2 <- NULL
    try(f1 <- fitdistrplus::fitdist(y, "lnorm"), silent = TRUE)
    try(f2 <- fitdistrplus::fitdist(y, "shiftlognormal",
                      start=list(mean=as.numeric(f0$estimate["meanlog"]),
                                 sigma=as.numeric(f0$estimate["sdlog"])),
                      fix.arg=list(shift=D)), silent = TRUE)

    if (is.null(f1)) {
      f <- f2
    } else if (is.null(f2)) {
      f <- f1
    } else if (f1$loglik > f2$loglik) {
      f <- f1
    } else {
      f <- f2
    }
  }
  f
}




fitShiftedLogStudent <- function(y, fallback=T) {
  f.ln_org <- f.ln <- fitShiftedLogNormal(y, fallback=F)
  D <- mean(y) - 2*min(y)
  start <- list(mean=log(mean(y+D)), sigma=sd(log(y+D)), shift=D, nu=30)

  if (!is.null(f.ln)) {
    start <- as.list(f.ln$estimate)
    start[["nu"]] <- 30
  }

  f <- NULL
  try(
    f <- fitdistrplus::fitdist(y, "shiftlstudent", start=start),
    silent = TRUE)

  if (!fallback || (!is.null(f) && !any(is.na(f$sd)))) {
    if ((f$estimate["nu"] > 100 || is.na(f$sd["nu"])) && !is.null(f.ln)) return(f.ln)
    return(f)
  }

  if (is.null(f) && is.null(f.ln)) return(fitShiftedLogNormal(y))

  if (is.null(f.ln)) {
    f.ln <- f
    f.ln$estimate <- f.ln$estimate[ which(names(f.ln$estimate) != "nu") ]
  }

  for (i in 1:10) {
    f.nu <- NULL
    try(
      f.nu <- fitdistrplus::fitdist(y, "shiftlstudent", start=list(nu=30), fix.arg = as.list(f.ln$estimate)),
      silent = T
    )
    if (is.null(f.nu)) break
    start <- as.list(f.ln$estimate)
    f.ln <- NULL
    try(
      f.ln <- fitdistrplus::fitdist(y, "shiftlstudent", start=start, fix.arg = as.list(f.nu$estimate), optim.method="BFGS")
      , silent=TRUE)
    if (is.null(f.ln)) break
  }

  stopifnot(!is.null(f.nu) || !is.null(f.ln))
  f <- f.ln
  if (is.null(f.ln)) { f <- f.nu }
  if (is.null(f.ln_org)) return(f)
  if (f$aic < f.ln_org$aic) {
    return(f)
  } else {
    return(f.ln_org)
  }
}


AUC_to_pValue <- function(r_aucs, do.plot=F, add.density=F) {
  d <- (r_aucs %>% filter(!control))$AUC
  d <- d[ d > lower_AUC_cutoff_inactive_compounds & d < upper_AUC_cutoff_inactive_compounds ]

  f <- fitStudentOrNormal(d)
  if (is.na(f$estimate["nu"])) {
    r_aucs$pv <- pnorm(r_aucs$AUC, f$estimate["mean"], f$estimate["sd"])
    r_aucs$pv_upper <- pnorm(r_aucs$AUC, f$estimate["mean"], f$estimate["sd"], lower.tail = F)
    if (do.plot) {
      stop("Didn't implement plotting function for normal distribution")
    }
  } else {
    r_aucs$pv <- pstudent(r_aucs$AUC, f$estimate["nu"], f$estimate["mean"], f$estimate["sigma"])
    r_aucs$pv_upper <- pstudent(r_aucs$AUC, f$estimate["nu"], f$estimate["mean"], f$estimate["sigma"], lower.tail = F)

    if (add.density | do.plot) {
      r_aucs$density <- dstudent(r_aucs$AUC, f$estimate["nu"], f$estimate["mean"], f$estimate["sigma"])
    }

    if (do.plot) {
      r_aucs$density <- dstudent(r_aucs$AUC, f$estimate["nu"], f$estimate["mean"], f$estimate["sigma"])
      binwidth <- 0.05
      p <- ggplot(r_aucs, aes(AUC)) + geom_histogram(binwidth = binwidth, boundary = 0)
      p <- p + geom_line(aes(AUC, sum(r_aucs$AUC > lower_AUC_cutoff_inactive_compounds)*binwidth*density), color="red")
      p <- p + ggtitle(paste0("Actual AUCs vs. estimated density for inactive compounds (", r_aucs[1,"replicate_id"], ")"))
      print(p)

      p <- ggplot(r_aucs, aes(AUC, pv)) + geom_line()
      p <- p + ggtitle(paste0("Conversion between AUC and p-value (", r_aucs[1,"replicate_id"], ")"))
      p <- p + scale_y_log10(breaks=c(0.001, 0.01, 0.1, 1))
      print(p)

    }
  }
  r_aucs
}

AUC_to_pValue_by_plate <- function(r_aucs, do.plot=F, add.density=F, return.estimates = F) {

  d <- (r_aucs %>% filter(control == "control drug"))$normAUC

  f <- fitStudentOrNormal(d)
  ## ONLY FOR EXAMPLE PLOT: # f <- fitdistrplus::fitdist(d, "norm")

  if (return.estimates) return(fit2df(f, ""))

  if (is.na(f$estimate["nu"])) {
    r_aucs$pv <- pnorm(r_aucs$normAUC, f$estimate["mean"], f$estimate["sd"])
    r_aucs$pv_upper <- pnorm(r_aucs$normAUC, f$estimate["mean"], f$estimate["sd"], lower.tail = F)

    if (do.plot) {
      X <- seq(0, 1.3, 0.01)
      Y <- dnorm(X, f$estimate["mean"], f$estimate["sd"])
      N <- sum(r_aucs$control != "other drug")
      binwidth <- 0.05

      dd <- tibble(normAUC = X, count = N*binwidth*Y)

      p <- ggplot(r_aucs %>% filter(control == "control drug"), aes(normAUC, fill = control)) + geom_histogram(binwidth = binwidth, boundary = 0)
      p <- p + geom_line(data=dd, aes(normAUC, count), inherit.aes = F, linetype="dashed")
      p <- p + ggtitle(paste0("Actual AUCs vs. estimated density for inactive compounds (", r_aucs[1,"replicate_id"], ")"))
      p <- p + scale_x_continuous(breaks=seq(0, 2, 0.2)) + expand_limits(x=0)
      p <- p + theme_minimal() + theme(panel.grid = element_blank())
      print(p)

      # p <- ggplot(r_aucs, aes(normAUC, pv)) + geom_line()
      # p <- p + ggtitle(paste0("Conversion between AUC and p-value (", r_aucs[1,"replicate_id"], ")"))
      # p <- p + scale_y_log10(breaks=c(0.001, 0.01, 0.1, 1))
      # print(p)
    }

  } else {
    r_aucs$pv <- pstudent(r_aucs$normAUC, f$estimate["nu"], f$estimate["mean"], f$estimate["sigma"])
    r_aucs$pv_upper <- pstudent(r_aucs$normAUC, f$estimate["nu"], f$estimate["mean"], f$estimate["sigma"], lower.tail = F)

    if (add.density | do.plot) {
      r_aucs$density <- dstudent(r_aucs$normAUC, f$estimate["nu"], f$estimate["mean"], f$estimate["sigma"])
    }

    if (do.plot) {
      X <- seq(0, 1.3, 0.01)
      Y <- dstudent(X, f$estimate["nu"], f$estimate["mean"], f$estimate["sigma"])
      N <- sum(r_aucs$control != "other drug")
      binwidth <- 0.05

      dd <- tibble(normAUC = X, count = N*binwidth*Y)

      p <- ggplot(r_aucs, aes(normAUC, fill = control)) + geom_histogram(binwidth = binwidth, boundary = 0)
      p <- p + geom_line(data=dd, aes(normAUC, count), inherit.aes = F)
      p <- p + ggtitle(paste0("Actual AUCs vs. estimated density for inactive compounds (", r_aucs[1,"replicate_id"], ")"))
      p <- p + scale_x_continuous(breaks=seq(0, 2, 0.2)) + expand_limits(x=0)
      p <- p + theme_minimal() + theme(panel.grid = element_blank())
      print(p)

      # p <- ggplot(r_aucs, aes(pv, fill = control)) + geom_histogram(binwidth = 0.1, boundary=0)
      # p <- p + theme_minimal() + theme(panel.grid = element_blank())
      # print(p)

      # p <- ggplot(r_aucs, aes(normAUC, pv)) + geom_line()
      # p <- p + ggtitle(paste0("Conversion between AUC and p-value (", r_aucs[1,"replicate_id"], ")"))
      # p <- p + scale_y_log10(breaks=c(0.001, 0.01, 0.1, 1))
      # print(p)

    }
  }
  r_aucs
}

pfit <- function(v, f, lower.tail = T) {
  l <- f$estimate
  if (!is.null(f$fix.arg)) {
    l <- c(l, f$fix.arg)
  }

  if (lower.tail == "both") {
    list(
      do.call(paste0("p", f$distname), c(list("q" = v), l, lower.tail = TRUE)),
      do.call(paste0("p", f$distname), c(list("q" = v), l, lower.tail = FALSE))
    )
  }
  else
  {
    do.call(paste0("p", f$distname), c(list("q" = v), l, lower.tail = lower.tail))
  }
}



