#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(broom, stringr, readr, tibble, dplyr, ggplot2, ggrepel, viridis, tidyr, scales, forcats, gtable, mpoly, stringi, here)
})

source(here("processing_config.R"))

species_overview <- read_tsv(paste0(output_data_path, "species_overview.tsv"), col_types = cols(
  NT_code = col_character(),
  species_name = col_character(),
  strain = col_integer(),
  specI_cluster = col_character(),
  species = col_character(),
  is_unique = col_logical(),
  species_label = col_character(),
  biosafety = col_character()
))

combined_pv <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))

combined_pv$veterinary <- str_detect(combined_pv$target_species, "^veterinary:")
combined_pv$human_use <- !str_detect(combined_pv$target_species, "^not a drug") & !combined_pv$veterinary

combined_hits <- combined_pv %>% group_by(target_species, prestwick_ID, human_use) %>% summarise(n_hit = sum(hit), hit = any(hit))

target_species_mapping5 <- tibble::frame_data(
  ~target_species, ~target_species_group5,
  "bacteria", "human-use drugs",
  "antiseptic", "human-use drugs",
  "fungi", "human-use drugs",
  "metazoan parasites", "human-use drugs",
  "protozoa", "human-use drugs",
  "viruses", "human-use drugs",
  "human", "human-use drugs",
  "veterinary: infections", "vet",
  "veterinary: host animal", "vet",
  "veterinary: metazoan parasites", "vet",
  "not a drug", "non-drugs"
)

target_species_mapping3 <- tibble::frame_data(
  ~target_species, ~target_species_group3,
  "bacteria", "antibacterial drugs",
  "antiseptic", "antibacterial drugs",
  "fungi", "anti-infectives",
  "metazoan parasites", "anti-infectives",
  "protozoa", "anti-infectives",
  "viruses", "anti-infectives",
  "human", "human-targeted drugs"
) %>% mutate(target_species_group3 = factor(target_species_group3, levels = c("antibacterial drugs", "human-targeted drugs", "anti-infectives")))

target_species_mapping6 <- tibble::frame_data(
  ~target_species, ~target_species_group6,
  "bacteria", "antibacterial drugs",
  "antiseptic", "antibacterial drugs",
  "fungi", "anti-infectives",
  "metazoan parasites", "anti-infectives",
  "protozoa", "anti-infectives",
  "viruses", "anti-infectives",
  "human", "human-targeted drugs",
  "veterinary: infections", "vet",
  "veterinary: host animal", "vet",
  "veterinary: metazoan parasites", "vet",
  "not a drug", "other"
)

target_species_mapping7 <- tibble::frame_data(
  ~target_species, ~target_species_group7,
  "bacteria", "antibacterial drugs",
  "antiseptic", "antibacterial drugs",
  "fungi", "anti-infectives",
  "metazoan parasites", "anti-infectives",
  "protozoa", "anti-infectives",
  "viruses", "anti-infectives",
  "human", "human-targeted drugs",
  "veterinary: infections", "antibacterial drugs",
  "veterinary: host animal", "human-targeted drugs",
  "veterinary: metazoan parasites", "anti-infectives",
  "not a drug", "other"
)

pretty_target_species_mapping <- tibble::frame_data(
  ~target_species, ~target_species_,
  "bacteria", "antibiotics",
  "antiseptic", "antiseptics",
  "fungi", "antifungals",
  "metazoan parasites", "antiparasitics",
  "protozoa", "antiprotozoals",
  "viruses", "antivirals",
  "human", "human-targeted drugs",
  "veterinary: infections", "vet: anti-infectives",
  "veterinary: host animal", "vet: animal-targeted drugs",
  "veterinary: metazoan parasites", "vet: antiparasitics",
  "not a drug", "non-drugs"
)

hit_stats1 <- combined_hits %>% ungroup() %>% do(
  data.frame(
    effect = c("hit", "no hit"),
    n = c( sum(.$hit), sum(!.$hit)  )
  )
) %>% as.tbl()

hit_stats1$label <- "Prestwick Chemical Library"
hit_stats1$ymin <- 5

hit_stats2 <- combined_hits %>% left_join(target_species_mapping5, by="target_species") %>%
  group_by(target_species_group5) %>% do(
    data.frame(
      effect = c("hit", "no hit"),
      n = c( sum(.$hit), sum(!.$hit)  )
    )
  ) %>% ungroup() %>% rename(label = target_species_group5) %>% as.tbl()

hit_stats2$ymin <- 3

hit_stats3 <- combined_hits %>% filter(human_use) %>% left_join(target_species_mapping3, by="target_species") %>%
  group_by(target_species_group3) %>% do(
    data.frame(
      effect = c("hit", "no hit"),
      n = c( sum(.$hit), sum(!.$hit)  )
    )
  ) %>% ungroup() %>% rename(label = target_species_group3) %>% as.tbl()

hit_stats3$ymin <- 1

label_levels <- c("Prestwick Chemical Library", "human-use drugs", "non-drugs","vet", "antibacterial drugs", "human-targeted drugs", "anti-infectives")
hit_stats <- bind_rows(hit_stats1, hit_stats2, hit_stats3) %>% mutate(label = factor(label, levels = label_levels)) %>%
  arrange(label)

hit_stats$ymax <- hit_stats$ymin + 1

hit_stats <- hit_stats %>% group_by(ymin) %>% do({
  d <- .
  d$right <- cumsum(d$n)
  d$left <- c(0, d$right[1:(nrow(d)-1)])
  spacing <- 40 * rep(seq(0, (nrow(d)/2-1)), each = 2)
  d$left <- d$left + spacing
  d$right <- d$right + spacing
  d
})

hit_stats <- hit_stats %>% group_by(label, ymin) %>%
  mutate(N_cumul = cumsum(n),
         N_total = sum(n),
         percentage = paste0(round(100*N_cumul / N_total, 0), "%"))

hit_stats_only_hits <- hit_stats %>% ungroup() %>% filter(ymin != max(ymin)) %>%
  group_by(ymin) %>% arrange(effect, label) %>%  do({
    d <- .
    d$prev_right <- cumsum(d$n)
    d$prev_left <- c(0, d$prev_right[1:(nrow(d)-1)])
    d
  }) %>% mutate(label = paste(label, effect))

hit_stats_cat <- hit_stats %>% group_by(label, ymin, ymax) %>% summarise(left = min(left), right=max(right))
hit_stats_cat$effect <- "hit"
hit_stats_cat <- hit_stats_cat %>% ungroup() %>% mutate(label = recode(label, "antibacterial drugs" = "antibacterials"))

hit_poly <- hit_stats_only_hits %>% rowwise() %>% do({
  d <- frame_data(~x, ~y,
                  .$left, .$ymax,
                  .$right, .$ymax,
                  .$prev_right, .$ymax + 1,
                  .$prev_left, .$ymax + 1
  )
  d$label <- .$label
  d$effect <- .$effect
  d$ymin <- .$ymin
  d
})

hit_stats$hjust <- ifelse(hit_stats$label == "vet", ifelse(hit_stats$effect == "hit", 1, 0),
                          ifelse(hit_stats$label == "antibacterial drugs", ifelse(hit_stats$effect == "hit", 1, 0.5),
                          0.5))


combined_pv_sp <- combined_pv %>% left_join(target_species_mapping3, by="target_species") %>%
  filter(human_use) %>%
  left_join(species_overview, by="NT_code") %>% group_by(prestwick_ID) %>% mutate(any_hit = any(hit)) %>%
  ungroup() %>% filter(any_hit)

# combined_pv_sp$target_species_group2 <- str_replace(combined_pv_sp$target_species_group2, " ", "\n")

combined_pv_sp$species_label <- ordered(combined_pv_sp$species_label,
                                        levels = (combined_pv_sp %>% group_by(species_label) %>%
                                                    summarise(N_hit = sum(hit)) %>% arrange(-N_hit))$species_label)

combined_pv_sp$prestwick_ID <- factor(combined_pv_sp$prestwick_ID,
                                      levels = (combined_pv_sp %>% group_by(target_species_group3, prestwick_ID) %>%
                                                  summarise(N_hit = sum(hit), first_species = min(species_label[hit])) %>% arrange(target_species_group3, -N_hit))$prestwick_ID)

combined_pv_sp$any_hit <- NULL
combined_pv_sp$effect <- "no hit"
combined_pv_sp[ combined_pv_sp$hit, ]$effect <- "hit"

scale_x <- 1.35
scale_y <- -0.5

combined_pv_sp <- combined_pv_sp %>% mutate(
  left = 200 + (as.integer(prestwick_ID)-1)*scale_x + (as.integer(target_species_group3)-1)*20,
  right = left + scale_x,
  ymax = as.integer(species_label) * scale_y,
  ymin = ymax + scale_y)

hit_poly2 <- bind_rows(
  hit_poly,
  combined_pv_sp %>% group_by(target_species_group3) %>% do({
    data_frame(y = max(.$ymax), x = c(min(.$left), max(.$right)))
  }) %>% rename(label = target_species_group3) %>% ungroup() %>% mutate(label = paste("H", label, "hit")) %>% arrange(-x) %>% mutate(effect = "hit"),
  hit_poly %>% ungroup() %>% filter(y == min(y), effect == "hit") %>% mutate(y = y-1, label = paste("H", label)) %>% select(-ymin)
)

species_labels <- combined_pv_sp %>% filter(right == max(right)) %>% select(ymin, ymax, right, species_label, NT_code) %>% arrange(ymin)

full_abundances <- read_tsv("annotations/tax_info_screen_specI_clusters.txt", col_types = cols())
full_abundances <- species_labels %>% full_join(full_abundances, by="NT_code")

abundances <- full_abundances %>% mutate(
  Rel_ab_median = ifelse(is.na(Rel_ab_median), 0, Rel_ab_median),
  Rel_ab_p25 = ifelse(is.na(Rel_ab_p25), 0, Rel_ab_p25),
  Rel_ab_p75 = ifelse(is.na(Rel_ab_p75), 0, Rel_ab_p75)
)

p_abundances <-
  ggplot(abundances, aes(middle = Rel_ab_median+1e-6, lower = Rel_ab_p25+1e-6, upper = Rel_ab_p75+1e-6,
                         ymin = Rel_ab_p25+1e-6, ymax = Rel_ab_p75+1e-6, width = 0.5,
                         fct_rev(species_label))) +
  # geom_errorbarh(aes(xmin = Rel_ab_p25+1e-6, xmax = Rel_ab_p75+1e-6), height = 0) +
  # geom_point(size = 1, color = "white") + geom_point( size = 0.25) +
  geom_boxplot(stat="identity") + coord_flip() +
  scale_y_log10(breaks=10**seq(-5, -1, 2), labels=c("1e-5", "0.001", "0.1")) +
  theme_minimal() + ylab("Relative abundance") +
  theme(panel.grid.major.y = element_blank(), axis.text.x = element_text(size = 6), axis.title.x = element_text(size = 7))

H_ymin <- min(species_labels$ymin)
H_ymax <- max(species_labels$ymax)
B_xmax <- max(hit_stats$right)
B_xmin <- min(combined_pv_sp$left)-20

FE_levels <- c("hit: antibacterial drugs", "hit: human-targeted drugs", "hit: anti-infectives", "no hit", "other hit")

hit_stats$fill_effect <- factor(ifelse(hit_stats$effect == "hit",
                                ifelse(hit_stats$label %in% c("antibacterial drugs", "human-targeted drugs", "anti-infectives"), paste0("hit: ", hit_stats$label), "other hit"),
                                "no hit"
), levels = FE_levels)


combined_pv_sp$fill_effect <- factor(ifelse(combined_pv_sp$effect == "hit",
                                     ifelse(combined_pv_sp$target_species_group3 %in% c("antibacterial drugs", "human-targeted drugs", "anti-infectives"), paste0("hit: ", combined_pv_sp$target_species_group3), "other hit"),
                                     "no hit"
), levels = FE_levels)

hit_counts <- combined_pv_sp %>% filter(hit) %>% group_by(species_label, fill_effect) %>% summarise(n=n())

p_counts <- ggplot(hit_counts, aes(fct_rev(species_label), n, fill=fct_rev(fill_effect))) + geom_bar(stat = "identity") +
  scale_fill_manual(values = c(dbcolors$ai_hit, dbcolors$ht_hit, dbcolors$ab_hit)) + coord_flip() + theme_minimal() + ylab("Number of drugs") +
  theme(panel.grid.major.y = element_blank(),
        axis.text.x = element_text(size = 6),  axis.title.x = element_text(size = 7)) +
  scale_x_discrete(expand=c(0,0))

hit_poly2 <- hit_poly2 %>% ungroup() %>% group_by(label) %>% mutate(plotbezier = max(x)-min(x) > 500 & effect == "hit" &
                                                                      !stri_startswith_fixed(label, "H"))

hit_poly2a <- hit_poly2 %>% filter(!plotbezier)
hit_poly2b <- hit_poly2 %>% filter(plotbezier)

getBezier <- function(df) {

  maxy <- max(df$y)
  miny <- min(df$y)

  w <- df[2,]$x - df[1,]$x

  pts <- df %>% group_by(y) %>%
    mutate(x = mean(x)) %>% ungroup() %>%
    mutate(y = c(miny, maxy, miny, maxy))

  f <- bezierFunction(pts[,c("x", "y")])

  b <- unique(as.tbl(as.data.frame(f(
    c( seq(0, 0.1, 0.001), seq(0.1, 0.9, 0.01), seq(0.9, 1, 0.001) )
  ))))

  h <- 0.2*w/50+0.2
  b$ynorm <- (b$y - miny)/(maxy - miny)

  bind_rows(
    b %>% mutate(x = x - w/2, y = y - h*(0.5**2 - (0.5-ynorm)**2)),
    b %>% mutate(x = rev(x+w/2), y = rev(y + h*(0.5**2 - (0.5-ynorm)**2) )
    ))
}

hit_poly2bx <- hit_poly2b %>% group_by(label, effect) %>% filter(n() == 4) %>% do(getBezier(.))

hit_stats_label <- hit_stats %>% filter(effect == "hit", n>40) %>%
  mutate( percentage = ifelse(label == "Prestwick Chemical Library",
#                             paste(percentage, "of drugs reduced growth"),
                              paste(percentage, "with anticommensal activity"),
                              percentage))

hit_stats_bg <- hit_stats %>% group_by(label, ymin, ymax) %>%
  summarise(left = min(left), right = max(right))

# enumerate territories of adjacent items that have the same value
# A A B B A C --> 1 1 2 2 3 4
enumAreas <- function(v) {
  idx <- 1 : (length(v) - 1)
  c(0, cumsum(v[idx] != v[idx+1]))
}

combined_pv_sp_coalesced <- combined_pv_sp %>% select(NT_code, fill_effect, left, right, ymin, ymax, target_species_group3) %>% group_by(NT_code, target_species_group3) %>%
    arrange(left) %>% mutate(area = enumAreas(fill_effect)) %>% group_by(area, NT_code, fill_effect, ymin, ymax, target_species_group3) %>%
    summarise(left = min(left), right = max(right))

hit_poly2a_hit <- hit_poly2a %>% filter(effect == "hit")
hit_poly2a_nonhit <- hit_poly2a %>% filter(effect != "hit")

species_labels <- species_labels %>% mutate(label = stri_replace_all_regex(species_label, "^([^ ]+) ([^ ]+)", "italic('$1_$2')"),
                          label = stri_replace_all_fixed(label, " ", "~"),
                          label = stri_replace_all_fixed(label, "_", " "))

p <- ggplot(hit_stats, aes(xmin = left, xmax = right, ymin = ymin, ymax = ymax, fill = fill_effect)) +
  geom_polygon(data = bind_rows(hit_poly2a_nonhit), aes(x=x, y=y, group=label), inherit.aes = F, fill = "#eeeeee") +
  geom_polygon(data = bind_rows(hit_poly2a_hit, hit_poly2bx), aes(x=x, y=y, group=label), inherit.aes = F, fill = "#bbbbbb") +
  geom_rect(data = hit_stats_bg, fill = "black", color = "black") +
  geom_rect() +
  geom_text(aes(x = right, y = ymin, label = N_cumul, hjust = hjust), angle=0, vjust = 1.2, size = 2) +
  geom_text(aes(x = (right+left)/2, y = ymax, label = label), vjust = 0, nudge_y = 0.1, color = "black", data = hit_stats_cat, size = 2.5, inherit.aes = F) +
  geom_text(aes(x = (right+left)/2, y = (ymin+ymax)/2, label = percentage), color = "white", data = hit_stats_label, size = 2) +
  ylab("") + xlab("") +
  scale_y_continuous(expand = c(0,0), limits = c(min(combined_pv_sp$ymax)-0.5, 7)) +
  scale_fill_manual(values = c(dbcolors$ab_hit, dbcolors$ht_hit, dbcolors$ai_hit, dbcolors$no_hit, dbcolors$any_hit)) +
  scale_x_continuous(limits = c(-10, B_xmax+25), expand = c(0,0)) +
  theme_minimal() +
  theme(axis.text = element_blank(), panel.grid = element_blank(), legend.position = "none") +
  geom_rect(data=combined_pv_sp_coalesced) +
  annotation_custom(gtable_filter(ggplotGrob(p_abundances), "panel"), xmin = 1060, xmax = B_xmax, ymin = H_ymin, ymax = H_ymax) +
  annotation_custom(gtable_filter(ggplotGrob(p_abundances), "axis-b"), xmin = 1060, xmax = B_xmax, ymax = H_ymin+scale_y) +
  annotation_custom(gtable_filter(ggplotGrob(p_abundances), "xlab"), xmin = 1060, xmax = B_xmax, ymin = H_ymin+2*scale_y, ymax = H_ymin) +
  annotation_custom(gtable_filter(ggplotGrob(p_counts), "panel"), xmin = 0, xmax = B_xmin, ymin = H_ymin, ymax = H_ymax) +
  annotation_custom(gtable_filter(ggplotGrob(p_counts), "axis-b"), xmin = 0, xmax = B_xmin, ymax = H_ymin+scale_y) +
  annotation_custom(gtable_filter(ggplotGrob(p_counts), "xlab"), xmin = 0, xmax = B_xmin, ymin = H_ymin+2*scale_y, ymax = H_ymin) +
  geom_text(data=species_labels, aes(x = right, y = (ymin+ymax)/2, label = label),
            inherit.aes = F, hjust = 0, nudge_x = 5, size = 2, parse = TRUE)

ggsave("figure_pdf/fig1a.pdf", p, width = 13, height = 15, units = "cm")

N_species <- length(unique(combined_pv$NT_code))

drug_counts <- combined_pv %>% left_join(target_species_mapping3, by = "target_species") %>% group_by(prestwick_ID, target_species, target_species_group3) %>%
  summarise(N_species = sum(hit))

category_counts <-
  drug_counts %>% group_by(target_species) %>% summarise(f_hit = sum(N_species > 0)/n()) %>%
    left_join(target_species_mapping6, by = "target_species") %>%
    left_join(target_species_mapping7, by = "target_species") %>%
    mutate(vet = target_species_group6 == "vet",
           target_species_group7 = fct_reorder(target_species_group7, f_hit, max)) %>%
    arrange(target_species_group7, !vet, f_hit)

category_summary <- drug_counts %>% ungroup() %>%
  left_join(target_species_mapping6, by = "target_species") %>%
  left_join(pretty_target_species_mapping, by = "target_species") %>%
  # mutate(target_species_ = str_replace(target_species, "veterinary", "vet")) %>%
  group_by(target_species, target_species_, target_species_group6, N_species) %>%
  summarise(N = n()) %>% arrange(target_species, N_species) %>% ungroup() %>%
  mutate(target_species = factor(target_species, levels = category_counts$target_species)) %>%
  group_by(target_species, target_species_, target_species_group6) %>% mutate(N_drugs_total = sum(N)) %>%
  filter(N_species > 0) %>% arrange(N_species) %>%
  mutate(N_rel = N/N_drugs_total, right = cumsum(N_rel), left = right - N_rel, y = as.integer(target_species))

category_outline <- category_summary %>% group_by(y, target_species, target_species_, target_species_group6) %>%
  summarise(left = min(left), right = max(right), label = paste0(sum(N), "/", N_drugs_total[1]))


minX <- -1.2
X <- 1.2

ggplot(category_summary, aes(xmin = left, xmax = right, ymin = y-0.25, ymax = y+0.25, fill = N_species)) +
  # geom_bar(data = category_summary %>% mutate(N_rel = 1.0) %>% unique(), aes(fill = target_species_group3),
  #          stat="identity", width = 0.5) +
  # geom_bar(data = category_outline, size = 1, color = "black", stat="identity", width = 0.5) +
  geom_rect(data = category_outline %>% filter(target_species_group6 == "antibacterial drugs"),
            aes(xmin = minX, xmax = X, ymin = y-0.5, ymax = y+0.5), fill = dbcolors$ab_hit) +
  geom_rect(data = category_outline %>% filter(target_species_ == "vet: anti-infectives"),
            aes(xmin = minX, xmax = X, ymin = y-0.5, ymax = y+0.5), fill = dbcolors$ab_hit, alpha = 0.75) +
  geom_rect(data = category_outline %>% filter(target_species_group6 == "anti-infectives"),
            aes(xmin = minX, xmax = X, ymin = y-0.5, ymax = y+0.5), fill = dbcolors$ai_hit) +
  geom_rect(data = category_outline %>% filter(target_species_ == "vet: antiparasitics"),
            aes(xmin = minX, xmax = X, ymin = y-0.5, ymax = y+0.5), fill = dbcolors$ai_hit, alpha = 0.75) +
  geom_rect(data = category_outline %>% filter(target_species == "human"),
            aes(xmin = minX, xmax = X, ymin = y-0.5, ymax = y+0.5), fill = dbcolors$ht_hit) +
  geom_rect(data = category_outline %>% filter(target_species_ == "vet: animal-targeted drugs"),
            aes(xmin = minX, xmax = X, ymin = y-0.5, ymax = y+0.5), fill = dbcolors$ht_hit, alpha = 0.75) +
  geom_rect(data = category_outline %>% filter(target_species == "not a drug"),
            aes(xmin = minX, xmax = X, ymin = y-0.5, ymax = y+0.5), fill = dbcolors$any_hit, alpha = 0.75) +
  # geom_vline(data=data_frame(x=c(0,0.5,1)), aes(xintercept = x), size = 0.1) +
  geom_rect(data = category_outline, size = 1, color = "black") +
  geom_rect() +
  geom_text(data = category_outline, aes(x = right, y = y, label=label), hjust = 0, nudge_x = 0.05, size=2) +
  geom_text(data = category_outline, aes(x = left, y = y, label=target_species_), hjust = 1, nudge_x = -0.05, size=2) +
  scale_y_continuous(limits = c(0.5, 11.5), expand = c(0,0)) +
  scale_x_continuous(limits = c(minX, X), expand=c(0,0), breaks = c(0, 0.5, 1)) +
  scale_fill_gradient(name = "Number of\naffected strains", high="white") +
  ylab("") + xlab("Fraction of drugs with anticommensal activity") +
  theme_minimal() + theme(panel.grid = element_blank(), axis.text.y = element_blank(), axis.title.y = element_blank(),
    legend.position = "bottom",
    axis.text.x = element_text(size = 6),  axis.title.x = element_text(size = 7),
    legend.title = element_text(size = 7),
    legend.text = element_text(size = 6),
    legend.box.margin = margin(0),
    legend.margin = margin(-0.2, unit = "line"),
    legend.key.height = unit(0.75, "line")
  )

ggsave("figure_pdf/fig1b_drugs_by_target_species.pdf", width = 6, height = 7.5, units = "cm")
# ggsave("figure_pdf/drugs_by_target_species.png", width = 3.5, height = 4)

# res <- 150
# ggsave("figure_pdf/drugs_by_target_species.png", width = 1024/res, height = 768 / res)


abundances$abundance_known <- !is.na(abundances$Rel_ab_median) & abundances$Rel_ab_median > 0

suppressWarnings(species_annotations <- read_tsv("annotations/species_annotations.tsv", col_types = cols(
  .default = col_character(),
  `Risk group` = col_integer()
)))

gram <- species_annotations %>% select(NT_code, Gram_stain)


abundances <- abundances %>% left_join(gram, by = "NT_code")

abx_human_long <- combined_pv %>% left_join(target_species_mapping3, by = "target_species") %>%
  group_by(NT_code, target_species_group3) %>% summarise(N = sum(hit)) %>% filter(!is.na(target_species_group3))

abx_human_long_abundance <- abx_human_long %>% left_join(abundances %>% select(species_label, NT_code, Rel_ab_median, Gram_stain), by = "NT_code")
abx_human_long_abundance$abundance_known <- !is.na(abx_human_long_abundance$Rel_ab_median) & abx_human_long_abundance$Rel_ab_median > 0


ab_caption <- abx_human_long_abundance %>% filter(abundance_known) %>% group_by(target_species_group3) %>%
  summarise(corr_spearman = cor(Rel_ab_median, N, method = "s"),
            pv_corr_spearman = cor.test(Rel_ab_median, N, method = "s")$p.value,
            y = mean(N)) %>%
  ungroup() %>%
  mutate(label1 = target_species_group3,
         label2 = paste0("r[s] == ", round(corr_spearman, 2), "~(p == ", sprintf("%.2g", pv_corr_spearman),")"))

xmin <- min((abx_human_long_abundance %>% filter(abundance_known))$Rel_ab_median)

ab_caption$y <- c(108, 53, -5)

example <- abx_human_long_abundance %>% ungroup() %>% filter(Rel_ab_median == max(Rel_ab_median)) %>%
  group_by(species_label, Rel_ab_median) %>%
  summarise(y = min(N)-15, yend = max(N)) %>% ungroup() %>%
  mutate(species_label = stri_replace_all_fixed(species_label, " ", "\n"))


ggplot(abx_human_long_abundance %>% filter(abundance_known),
       aes(Rel_ab_median, N, color = target_species_group3)) +
  geom_smooth(method="lm", alpha = 0.1, size=0.5) +
  # geom_segment(data = example, aes(x=Rel_ab_median, xend=Rel_ab_median, y=y, yend=yend), inherit.aes=F, size = 0.25)+
  # geom_text(data = example, aes(x=Rel_ab_median*1.1, y=y-2, label = species_label), inherit.aes=F,
  #           hjust = "inward", vjust = "top", fontface="italic", lineheight = 1, size = 2*5/7)+
  geom_point(size=0.5) + scale_x_log10() +
  geom_text(data = ab_caption, aes(x = xmin, y = y, label = label1), hjust = 0, vjust = -0.2, parse=F, size = 2) +
  geom_text(data = ab_caption, aes(x = xmin, y = y, label = label2), hjust = 0, vjust = 1.2, parse=T, size = 2) +
  scale_color_manual(values = c(dbcolors$ab_hit, dbcolors$ht_hit, dbcolors$ai_hit)) +
  theme_minimal() + ylim(-10,110) +
  theme(legend.position = "none") +
  xlab("Relative abundance") +
  ylab("Number of drugs with anticommensal activity per strain") +
  ylab("Number of drugs with anticommensal activity per strain") +
  theme(axis.text = element_text(size = 6),  axis.title = element_text(size = 7))

ggsave("figure_pdf/fig1c_abundance.pdf", width = 6, height = 7, units = "cm")
