#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(tidyverse, ggstance, here, stringi)
})

source(here("processing_config.R"))

# output_data_path <- "/g/bork5/mkuhn/data/drugbug/output/"

species_overview <- read_tsv(paste0(output_data_path, "species_overview.tsv"), col_types = cols(
  NT_code = col_character(),
  species_name = col_character(),
  strain = col_integer(),
  specI_cluster = col_character(),
  species = col_character(),
  is_unique = col_logical(),
  species_label = col_character(),
  biosafety = col_character()
))

combined_pv <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))

combined_pv$veterinary <- str_detect(combined_pv$target_species, "^veterinary:")
combined_pv$human_use <- !str_detect(combined_pv$target_species, "^not a drug") & !combined_pv$veterinary

combined_hits <- combined_pv %>% group_by(target_species, prestwick_ID, human_use) %>% summarise(n_hit = sum(hit), hit = any(hit))

concs <- read_tsv("annotations/drugbank_elimination_processed.tsv")

concentrations <- bind_rows(
  read_tsv(paste0(output_data_path, "LM17-29-concentrations.tsv")),
  read_tsv(paste0(output_data_path, "LM16-121_142-concentrations.tsv"))
)

MICs <- bind_rows(
  read_tsv(paste0(output_data_path, "LM17-29-MIC.tsv")) %>% filter(stri_detect_fixed(Drug, "azole")) %>% filter(NT_code != "NT5031"),
  read_tsv(paste0(output_data_path, "LM16-121_142-MIC.tsv")) %>% filter(stri_startswith(prestwick_ID, fixed="Prest"), Drug != "Metformin")
)


### NUMBER FOR PAPER
MICs %>% select(Drug) %>% unique() %>% nrow()
MICs %>% select(NT_code) %>% unique() %>% nrow()

MICs_ <- MICs

MICs <- MICs %>% left_join(combined_pv, by = c("NT_code", "prestwick_ID"))

## output for MIC table
m <- MICs %>% filter(!(Conc < 40 & qualifier == ">")) %>%
  mutate(binConc = ifelse(Conc < 10, "< 10", ifelse(Conc > 40, "> 40", Conc))) %>%
  group_by(hit, binConc) %>% count() %>% reshape2::acast(binConc ~ hit, value.var = "n")

### RAW DATA FOR SUPPL TABLE
m[c("< 10", "10", "20", "40", "> 40"),]

MICs <- MICs %>% mutate(concBin = factor(ifelse(Conc < 10, "< 10", ifelse(Conc > 40, "> 40", "10 - 40")), levels = rev(c("< 10", "10 - 40", "> 40")))) %>%
  mutate(result2 = paste(hit, concBin, qualifier != ">"),
         result = recode(result2,
                         # qualifier either < or =
                         "TRUE < 10 TRUE" = "TP",
                         "TRUE 10 - 40 TRUE" = "TP",
                         "TRUE > 40 TRUE" = "FP",
                         "FALSE < 10 TRUE" = "FN",
                         "FALSE 10 - 40 TRUE" = "TN",
                         "FALSE > 40 TRUE" = "TN",
                         # qualifier >
                         "TRUE 10 - 40 FALSE" = "FP",
                         "TRUE > 40 FALSE" = "FP",
                         "FALSE 10 - 40 FALSE" = "TN",
                         "FALSE > 40 FALSE" = "TN"
         ))


MIC_cc <- MICs %>% left_join(species_overview, by="NT_code") %>% select(-CID_flat) %>%
  left_join(read_tsv(paste0(output_data_path, "compound_properties.tsv"))) %>%
  mutate( MIC25_ug_ml = Conc / 1000 * MolecularWeight, MIC90_ug_ml = Conc90 / 1000 * MolecularWeight ) %>%
  select(chemical_name, prestwick_ID, species_label, NT_code, result, qualifier, Conc, MIC25_ug_ml, qualifier90, Conc90, MIC90_ug_ml) %>%
  rename( MIC25_uM = Conc, qualifier25 = qualifier, MIC90_uM = Conc90 )

tp2 <- MICs %>% group_by(prestwick_ID, Conc, Drug, chemical_name, result, qualifier) %>% summarise(N=n()) %>%
  group_by(Conc, prestwick_ID, qualifier) %>% arrange(result) %>%
  mutate(N_cumul = cumsum(N))

drug_order <- (MICs %>% group_by(Drug) %>% summarise(Conc = mean(log10(Conc))) %>% arrange(Conc))$Drug

qualifier_shift <- tribble(~qualifier, ~shift,
                              "<", 0.8,
                              "=", 1,
                              ">", 1/0.8)

tp2 <- tp2 %>% left_join(qualifier_shift, by = "qualifier")
tp2 <- tp2 %>% group_by(Conc, Drug) %>% mutate(multi = length(unique(qualifier)) > 1)

tp2$sConc <- ifelse(tp2$multi, tp2$shift * tp2$Conc, tp2$Conc)

tp2 <- tp2 %>% ungroup() %>% mutate(Drug = factor(Drug, levels = drug_order), iDrug = as.integer(Drug))


concentrations <- concentrations %>% mutate(Drug = factor(Drug, levels = drug_order),
                                            iDrug = as.integer(Drug)) %>% filter(!is.na(Drug)) %>%
  inner_join(tp2 %>% select(prestwick_ID, chemical_name) %>% unique(), by="prestwick_ID")

b <- 20*2**seq(-8, 4, 2)
# b <- c(0.3125, 1.25, 5, 20, 80, 320)
lb <- as.character(b)

pal <-  c("green", "red", "grey", "blue" )[ c("FN", "FP", "TN", "TP") %in% unique(tp2$result) ]
shapes <- c(25, 21, 24)[ c("<", "=", ">") %in% unique(tp2$qualifier)  ]

tp_concs <- tp2 %>% select(prestwick_ID, Drug) %>% unique() %>% left_join(concs) %>%
  select(Drug, colon_conc_uM, plasma_conc_uM, intestine_conc_uM) %>%
  gather(kind, Conc, colon_conc_uM, plasma_conc_uM, intestine_conc_uM) %>% filter(!is.na(Conc)) %>%
  mutate(iDrug = as.integer(Drug))

ggplot(tp2 %>% arrange(-N_cumul), aes(Conc, Drug, size=N_cumul, color = result,  fill = result, shape = qualifier)) +
  geom_vline(xintercept = 20) +
  geom_rect(data = concentrations, aes(xmin = maxConc*1.5, xmax = Inf, ymin = iDrug - 0.5, ymax = iDrug + 0.5), inherit.aes = F,
            fill = "#eeeeee") +
  geom_rect(data = concentrations, aes(xmin = 0, xmax = minConc / 1.5, ymin = iDrug - 0.5, ymax = iDrug + 0.5), inherit.aes = F,
            fill = "#eeeeee") +
  geom_text(data = concentrations, aes(x = minConc/1.7, y = iDrug, label = chemical_name), inherit.aes = F, hjust = 1) +
  geom_point(aes(sConc, iDrug)) +
  # geom_text(data = tp_concs, aes(Conc, iDrug, label = stri_sub(kind, 1, 1)), inherit.aes = F) +
  # geom_point(data = tp2 %>% filter(result %in% c("TN", "TP")), aes(sConc, iDrug)) +
  # geom_point(data = tp2 %>% filter(result %in% c("FN", "FP")), aes(sConc, iDrug)) +
  scale_color_manual(values = pal) +
  scale_fill_manual(values = pal) +
  scale_shape_manual(values = shapes) +
  scale_size(name = "number of\nstrains", breaks = c(1,2,5,10), limits = c(0,max(tp2$N_cumul)), range = c(0, 6)) +
  scale_x_log10(limits = c(1e-2, 1e3), breaks = b, labels = lb) +
  scale_y_continuous(expand = c(0,0)) +
  xlab(expression(IC[25]*" per strain (µM)")) + ylab("") +
  theme_minimal() + theme(panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(),
                          axis.text.y = element_blank())

# res <- 125
# ggsave("MICs.png", width = 1024/res, height = 768 / res)
ggsave(paste0("figure_pdf/figED5c_MICs.pdf"), width = 12, height = 10)



MICs_concs <- MICs_ %>%
  right_join(combined_pv %>% select(prestwick_ID, NT_code, hit) %>% filter( prestwick_ID %in% MICs$prestwick_ID )
             , by = c("prestwick_ID", "NT_code")) %>%
  select(-chemical_name) %>%
  left_join(concs, by = "prestwick_ID") %>%
  select(NT_code, chemical_name, hit, qualifier, Conc, colon_conc_uM, plasma_conc_uM, intestine_conc_uM) %>%
  gather(kind, body_conc, colon_conc_uM, plasma_conc_uM, intestine_conc_uM) %>%
  filter(!is.na(body_conc), qualifier != ">" | is.na(qualifier))

MICs_counts <- MICs_concs %>% group_by(chemical_name, kind, hit) %>%
  summarise(n_MIC_below = sum(Conc <= body_conc, na.rm = T),
            n_MIC_above = sum(Conc > body_conc, na.rm = T),
            n_MIC_not_determined = sum(is.na(Conc)),
            n_hit = n()) %>%
  ungroup()

MICs_counts_ <- MICs_counts %>% filter(hit) %>%
  mutate(chemical_name = fct_reorder(chemical_name, n_hit)) %>%
  select(chemical_name, kind, n_MIC_below, n_MIC_above, n_MIC_not_determined) %>%
  gather(conc_to_MIC, count, n_MIC_below, n_MIC_above, n_MIC_not_determined) %>%
  mutate(conc_to_MIC = recode_factor(conc_to_MIC,
                                     n_MIC_not_determined = "MIC not determined",
                                     n_MIC_above = "MIC above body concentration",
                                     n_MIC_below = "MIC below body concentration"
                              ),
         kind = recode_factor(kind,
                              intestine_conc_uM = "Small intestine", plasma_conc_uM = "Plasma", colon_conc_uM = "Colon"))


MICs_counts_ %>% ggplot(aes(count, chemical_name, fill = conc_to_MIC)) + geom_barh(stat = "identity") + facet_grid(~kind) +
  scale_x_continuous(breaks = seq(0, 40, 10)) +
  scale_fill_manual(values = c("grey", "#af8dc3", "#7fbf7b"), name = "",
                    labels = c(
                      expression(IC[25]*" not determined"),
                      expression(IC[25]*" above body concentration"),
                      expression(IC[25]*" below body concentration")
                    )) + ylab("") +
  xlab("Count of affected strains per drug") +
  theme_minimal() + theme(panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(), legend.position = "bottom")

ggsave(paste0("figure_pdf/figED6_MIC_vs_conc.pdf"), width = 20, height = 10, units = "cm")

threshold_counts <- MICs %>% filter(qualifier != ">") %>% group_by(Drug) %>% summarise(Conc = min(Conc)) %>%
  group_by(Conc) %>% count() %>% ungroup() %>% mutate(n_below = cumsum(n))

conc_range <- log2(range(unique(threshold_counts$Conc)))
conc_breaks <- 2**seq(conc_range[1], conc_range[2])

tpb <- MICs %>% group_by(hit, concBin) %>% summarise(n = n())

tpb <- bind_rows(
  tpb,
  anti_join(expand.grid(hit = unique(MICs$hit), concBin = levels(MICs$concBin), n = 0), tpb, by = c("hit", "concBin"))
) %>% arrange(hit, concBin)


tpb <- tpb %>% group_by(hit) %>% mutate(f = n/sum(n)) %>%
  mutate(hit_ = ifelse(hit, "Drug reduced growth in screen in given strain", "No significant growth reduction in screen")) %>%
  mutate(p = paste0(round(100*f, 0), "%"))

tpb$prefix <- c("False negatives: ", "", "", "", "", "False positives: ")
tpb$group <- c("True negative", "True negative", "False negative", "False positive", "True positive", "True positive" )
# tpb$p <- paste0(tpb$prefix, tpb$p )

tps <- tpb %>% group_by(hit_, group) %>%
  summarise(
    p = paste0(group[1], "s:\n", sum(n)),
    y0 = min(as.integer(concBin)),
    y1 = max(as.integer(concBin)),
    y = (y0+y1)/2,
    x = max(tpb$n)
  )

ggplot(tpb, aes(n, concBin)) + geom_barh(stat = "identity", aes(fill=group)) +
  facet_wrap(~hit_, ncol = 1) +
  # geom_text(aes(label = p), hjust = 0, nudge_x = 1) +
  # scale_fill_manual(values = c("#ddaa99", "#ddaa99", dbcolors$no_hit, dbcolors$ht_hit )) +
  scale_fill_manual(values = c(dbcolors$no_hit, dbcolors$ht_hit, dbcolors$no_hit, dbcolors$ht_hit )) +
  scale_x_continuous(expand = c(0,0), limits = c(0, max(tpb$n + 70)), breaks=seq(0, 100, 25)) +
  ylab(expression(IC[25]*" per drug-strain pair (µM)")) +
  xlab("Count of drug-strain pairs") +
  geom_linerange(data = tps, aes(x=x+2, ymin = y0-0.45, ymax = y1+0.45), inherit.aes = F) +
  geom_text(data = tps, aes(x=x+7, y=y, label=p), hjust = 0, lineheight = 1, size = 2) +
  theme_minimal() +
  theme(legend.position = "none", #strip.background = element_rect(size = 0, fill = "lightgrey"),
        strip.text = element_blank(), # element_text(size = 7),
        panel.grid.major.y = element_blank(),
        panel.grid.minor.x = element_blank(),
        legend.title = element_text(size = 7),
        legend.text = element_text(size = 6),
        legend.margin = margin(1),
        legend.key.height = unit(0.75, "line"),
        axis.title = element_text(size = 7),
        axis.text = element_text(size = 6),
        plot.title = element_text(face = "bold", size = 8)
    )

ggsave("figure_pdf/fig2b_MICs_grouped.pdf", width = 7, height = 4, units = "cm")
