#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(tidyverse, stringi, here, ggstance, forcats, gtable, ggrepel, scales)
})


source(here("processing_config.R"))

# http://dx.doi.org/10.1136/gutjnl-2015-310861
d1 <- read_tsv(here("annotations/gutjnl-2015-310861supp4.tsv"))
# http://dx.doi.org/10.1136/gutjnl-2015-310376
d2_ <- read_tsv(here("annotations/gutjnl-2015-310376supp_table2.tsv"))

d2 <- d2_ %>%
  mutate( dutch_meta_coeff = unlist(lapply(stri_split_fixed(Coefficients, ";"), function(v) mean(as.double(v))) ),
          Taxonomy = stri_replace_all_fixed(Taxon, "|", ";")) %>%
  select(Taxonomy, NT_codes, dutch_meta_fdr = MetaQvalue, dutch_meta_coeff, dutch_dir = Direction) %>%
  filter(dutch_meta_fdr < 0.05)

d2 <- d2 %>% filter(!stri_detect_fixed(Taxonomy, "/"))

TaxLevels <- d1 %>% select(Taxonomy, Level) %>% mutate(nL = stri_count_fixed(Taxonomy, ";")) %>% select(nL, Level) %>% unique()

d2 <- d2 %>% mutate(nL = stri_count_fixed(Taxonomy, ";")) %>% left_join(TaxLevels, by="nL")


species_overview <- read_tsv(paste0(output_data_path, "species_overview.tsv"), col_types = cols(
  NT_code = col_character(),
  species_name = col_character(),
  strain = col_integer(),
  specI_cluster = col_character(),
  species = col_character(),
  is_unique = col_logical(),
  species_label = col_character(),
  biosafety = col_character()
))

combined_pv_ <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))

p2at <- read_tsv(paste0(output_data_path, "prestwick_atc.tsv"))

ppi <- p2at %>% filter(stri_startswith(ATC, fixed = "A02BC")) %>% select(prestwick_ID, chemical_name) %>% unique() %>% filter(!is.na(prestwick_ID))

d <- full_join(d1, d2, by = c("Taxonomy", "NT_codes", "Level"))

d <- d %>% filter(!is.na(NT_codes)) %>% rowwise() %>% do(data.frame(., NT_code = stri_split_fixed(.$NT_codes, " ")[[1]], stringsAsFactors = F))

d <- d %>% filter(NT_code %in% combined_pv_$NT_code) %>% mutate(Taxonomy = stri_replace_all_regex(Taxonomy, "^k__[^;]+;|.__", ""))

tt0 <- unique(d$Taxonomy)

# add prefixes until fixpoint
tt <- unique(c(tt0, stri_replace_all_regex(tt0, ";[^;]*$", "")))
tt <- unique(c(tt, stri_replace_all_regex(tt, ";[^;]*$", "")))

d <- bind_rows(d, data.frame(Taxonomy = tt[ !tt %in% tt0 ], NT_code = "dummy", twins_UK_dir = "pos"))

combined_pv <- bind_rows(combined_pv_, data.frame(NT_code = "dummy")) %>% inner_join(d, by = "NT_code")

combined_pv <- combined_pv %>% mutate(Level = ordered(Level, levels = c("Phylum", "Class", "Order", "Family", "Genus", "Species"))) %>%
  mutate( Taxonomy = fct_rev(Taxonomy) )

# combined_pv %>% filter(NT_code == "NT5069") %>% View()

b <- c(seq(2, 6, 2), 30)

combined_pv <- combined_pv %>% group_by(Taxonomy) %>% mutate(NT_codes = paste(unique(NT_code), collapse = " "))

MICs_full <- read_tsv(paste0(output_data_path, "LM17-29-MIC.tsv")) %>% right_join(ppi) %>% filter(NT_code != "NT5031")
MICs <- MICs_full %>% filter(qualifier == "=") %>% left_join(combined_pv_)
MICs_full <- MICs_full %>% left_join(combined_pv_)

MICs %>% arrange(Conc) %>% head()

MICs %>% ggplot(aes(pv_comb_fdr_BH, Conc)) + geom_point() + facet_wrap(~chemical_name) +
  geom_vline(xintercept = 0.01) + geom_hline(yintercept = 20) +
  scale_y_log10() + scale_x_log10() + geom_smooth(method="lm")

combined_pv_merged <- combined_pv %>%
  right_join(ppi, by = "prestwick_ID") %>%
  left_join(MICs_full) %>%
  group_by(NT_codes) %>% mutate(
  i = as.integer(Taxonomy),
  ymin = ifelse(is.na(NT_codes), i, min(i)),
  ymax = ifelse(is.na(NT_codes), i, max(i)),
  y = (ymin+ymax)/2
) %>% ungroup() %>% select(ymin, ymax, y, NT_code, prestwick_ID, pv_comb_fdr_BH, qualifier, Conc) %>% unique()

# combined_pv %>% right_join(ppi, by = "prestwick_ID") %>%
#   select(NT_code, prestwick_ID, chemical_name, pv_comb_fdr_BH, Taxonomy, NT_codes, Level, twins_UK_fdr, twins_UK_coeff, dutch_meta_fdr, dutch_meta_coeff) %>%
#   ungroup() %>%
#   arrange(Taxonomy) %>% write_tsv("figure_source_data/ED_for_3a.tsv")

boxy <- function(v, log=F) {
  if (log) vv <- log(v)
  qs <- quantile(vv, c(0.25, 0.5, 0.75))
  iqr <- qs[3] - qs[1]

  if (iqr == 0) {
    d <- data.frame( bmin = qs[1], bmid = qs[2], bmax = qs[3], lmin = qs[1], lmax = qs[3], n = length(v))
  } else {
    if (log) {
      d <- data.frame( bmin = exp(qs[1]), bmid = exp(qs[2]), bmax = exp(qs[3]), lmin = exp(qs[1] - 1.5*iqr), lmax = exp(qs[3] + 1.5*iqr), n = length(v))
    } else {
      d <- data.frame( bmin = qs[1], bmid = qs[2], bmax = qs[3], lmin = qs[1] - 1.5*iqr, lmax = qs[3] + 1.5*iqr, n = length(v))
    }

    within_range <- v[ which(v > d$lmin & v < d$lmax) ]

    d$lmax <- max(within_range)
    d$lmin <- min(within_range)
  }

  d
}

tree <- combined_pv %>% select(twins_UK_coeff, twins_UK_fdr, dutch_meta_coeff, dutch_meta_fdr, Taxonomy) %>% unique()

tree <- tree %>% rowwise() %>% mutate(taxa = stri_split_fixed(Taxonomy, ";"),
                                      nl = length(taxa),
                                      prefix = ifelse(nl == 1, "", paste0(paste0(rep("   ", nl - 2), collapse = ""), " \U2514")),
                                      label = paste(prefix, taxa[nl])) %>% arrange(Taxonomy)

tree <- tree %>% ungroup %>% mutate(label = fct_inorder(label))


combined_pv_merged %>% filter(pv_comb_fdr_BH < 0.01) %>% select(NT_code) %>% unique() %>% left_join(species_overview) %>%
  select(NT_code, species_label) %>% arrange(NT_code)

boxes <- combined_pv_merged %>% group_by(y, ymin, ymax) %>% filter(!is.na(pv_comb_fdr_BH)) %>% do(boxy(.$pv_comb_fdr_BH, log=T))
outliers <- combined_pv_merged %>% right_join(boxes, by="y") %>% filter(pv_comb_fdr_BH < lmin | pv_comb_fdr_BH > lmax | n < 10)

pb <- ggplot(boxes %>% filter(n>=10)) +
  geom_hline(data = boxes, aes(yintercept = y), color = "lightgrey", size = 0.25) +
  geom_vline(xintercept = 0.01) +
  geom_linerangeh(aes(y = y, xmin = lmin, xmax = bmin)) +
  geom_linerangeh(aes(y = y, xmin = bmax, xmax = lmax)) +
  geom_crossbarh(aes(y = y, xmin = bmin, x = bmid, xmax = bmax), width = 0.5, fill = "white") +
  geom_point(data = outliers, aes(x = pv_comb_fdr_BH, y = y), size = 0.5) +
  scale_x_log10() +
  scale_y_continuous( limits =c(0.5, nrow(tree) + 0.5), expand = c(0,0)) +
  xlab("q-value of\nanticommensal activity") +
  # scale_y_discrete(breaks = tree$Taxonomy) +
  # facet_grid(twins_UK_dir~., scales="free", space="free") +
  theme_minimal() + theme(panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(),
                          axis.text.x = element_text(size = 6),  axis.title.x = element_text(size = 7))

  # ggplot(combined_pv, aes(pv_comb_fdr_BH, Taxonomy)) +
  #   geom_vline(xintercept = 0.01) +
  #   geom_boxploth() + scale_x_log10() +
  #   xlab("q-value of anticommensal activity") +
  #   # scale_y_discrete(breaks = tree$Taxonomy) +
  #   # facet_grid(twins_UK_dir~., scales="free", space="free") +
  #   theme_minimal()




X0 <- -0.7
X1 <- 0
X2 <- 1
X3 <- 2
m <- 0.05

tree2 <- bind_rows(
  tree %>% select(label, fdr = twins_UK_fdr, coeff = twins_UK_coeff) %>% mutate(study = "Twins UK"),
  tree %>% select(label, fdr = dutch_meta_fdr, coeff = dutch_meta_coeff) %>% mutate(study = "Dutch cohort study (rescaled)", coeff = coeff*5)
) %>% arrange(fdr)

pt <- ggplot(tree2, aes(coeff, label, color = study)) +
  geom_hline(data = tree2 %>% filter(!is.na(coeff)), aes(yintercept = as.integer(label)), color="lightgrey", size = 0.25) +
  geom_point(aes(size = -log10(fdr))) +
  scale_x_continuous(expand = c(0.1, 0)) +
  scale_size_area(breaks = b, labels = 10**-b, name = "q-value in cohort study") +
  scale_color_manual(values = c("#b2df8a", "#1f78b4")) +
  geom_vline(xintercept = 0) +
  theme_minimal() +
  theme(axis.text.y = element_text(hjust = 0, size = 6), legend.position = "bottom",
        legend.title = element_text(size = 7),
        legend.text = element_text(size = 6),
        legend.box.margin = margin(0),
        legend.margin = margin(-0.2, unit = "line"),
        legend.key.height = unit(0.75, "line"),
        panel.grid.major.y = element_blank(), legend.box = "vertical",
        axis.text.x = element_text(size = 6),  axis.title.x = element_text(size = 7)) +
  scale_y_discrete(expand=c(0,0.5)) +
  xlab("Association coefficient\nin cohort study")

triangles <- boxes %>% filter(ymin!=ymax) %>% group_by(y) %>% do(
  data.frame( x = c(0,0,1), y = c(.$ymin, .$ymax, .$y), group = .$y)
)

pc <- ggplot() + scale_x_continuous(limits = c(0,1), expand = c(0,0)) + scale_y_continuous(limits = c(0.5,nrow(tree)+0.5), expand = c(0,0)) +
  geom_hline(data = boxes %>% filter(ymin==ymax), aes(yintercept = y), color="lightgrey", size = 0.25) +
  geom_polygon(data = triangles, aes(x = x, y = y, group = group), color="#eeeeee", fill="#eeeeee") +
  theme_void()

p <- ggplot() + theme_void() + xlim(X0, X3) + ylim(-5, nrow(tree)-1 ) + ggtitle("Effect of proton-pump inhibitors in vivo and in vitro") +
  annotation_custom(gtable_filter(ggplotGrob(pc), "panel"), xmin = X2-m, xmax = X2+m, ymin = 0.5, ymax = nrow(tree)+0.5) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "axis-l"), xmin = X0, xmax = X1-m, ymin = 0.5, ymax = nrow(tree)+0.5) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "panel"), xmin = X1, xmax = X2-m, ymin = 0.5, ymax = nrow(tree)+0.5) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "axis-b"), xmin = X1, xmax = X2-m, ymin = 0, ymax = 0.4) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "xlab"), xmin = X1, xmax = X2-m, ymin = -1, ymax = 0) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "guide-box"), xmin = X0, xmax = X3, ymin = -5, ymax = -5) +
  annotation_custom(gtable_filter(ggplotGrob(pb), "panel"), xmin = X2+m, xmax = X3, ymin = 0.5, ymax = nrow(tree)+0.5) +
  annotation_custom(gtable_filter(ggplotGrob(pb), "axis-b"), xmin = X2+m, xmax = X3, ymin = 0, ymax = 0.4) +
  annotation_custom(gtable_filter(ggplotGrob(pb), "xlab"), xmin = X2+m, xmax = X3, ymin = -1, ymax = 0) +
  theme(title = element_text(size = 8))

ggsave("figure_pdf/figED7a.pdf", p, width = 10, height = 14, units = "cm", device = Cairo::CairoPDF)


corrs <- combined_pv %>%
  group_by(prestwick_ID) %>% filter(any(hit)) %>%
  ungroup() %>% gather(study, coeff, dutch_meta_coeff, twins_UK_coeff) %>%
  select(NT_codes, prestwick_ID, pv_comb_fdr_BH, pv_comb, study, coeff) %>%
  filter(!is.na(coeff)) %>% unique() %>%
  group_by(NT_codes, study, prestwick_ID) %>%
  summarise(coeff = median(coeff), pv_comb_fdr_BH = median(pv_comb_fdr_BH), pv_comb = median(pv_comb) ) %>%
  group_by(study, prestwick_ID) %>%
  summarise( corr = cor(coeff, pv_comb_fdr_BH, method = "s"), corr_pv  = cor(coeff, pv_comb, method = "s") )

corrs <- corrs %>% full_join(ppi, by="prestwick_ID") %>%
  group_by(study) %>%
  mutate(is_ppi = prestwick_ID %in% ppi$prestwick_ID,
    corr_rank = rank(-corr)/n(),
    corr_pv_rank = rank(-corr_pv)/n())

corrs %>% filter(is_ppi)
corrs %>% filter(is_ppi) %>% group_by(study) %>% summarise(mean(corr_rank), mean(corr_pv_rank))

corrs <- corrs %>% ungroup() %>%
  mutate( study = recode(study,
                         "dutch_meta_coeff" = "Dutch cohort study",
                         "twins_UK_coeff" = "Twins UK study"),
          chemical_name = recode(chemical_name, "Rabeprazole Sodium salt" = "Rabeprazole"))

DX <- 0.075
DY <- 10

### corr_pv == correlation based on p-value, which is what we want to use here.

corr_triangles <-  corrs %>% ungroup() %>% filter(is_ppi) %>% mutate(chemical_name = fct_reorder(chemical_name, -corr_pv_rank)) %>%
  group_by(study) %>% arrange(corr_pv) %>% mutate( side = rep(c(-1, 1), length.out = n() )) %>%
  group_by(study, chemical_name) %>% do(
    data_frame(x = .$corr_pv + c(0, -DX, DX),
               y = c(0, DY, DY) * .$side,
               grouper = paste(as.character(.$study), as.character(.$chemical_name)))
  )

ggplot(corrs, aes(corr_pv)) + geom_histogram(binwidth = 0.2, boundary = 0, fill = dbcolors$ht_hit, alpha = 0.5) +
  facet_wrap(~study, ncol = 2) +
  geom_polygon(data = corr_triangles, aes(x = x, y = y, group = grouper, fill = chemical_name)) +
  geom_polygon(data = corr_triangles, aes(x = x, y = y, group = grouper, fill = chemical_name), alpha = 0.5) +
  theme_minimal() +
  theme(legend.position = "bottom",
        strip.background = element_rect(fill = "lightgrey", size = 0), panel.grid.minor = element_blank(),
        strip.text = element_text(size = 7),
        axis.text = element_text(size = 6),
        axis.title = element_text(size = 7),
        legend.title = element_text(size = 7),
        legend.text = element_text(size = 6)
  ) +
  scale_fill_brewer(palette = "Set1", name = "Drug") +
  scale_color_brewer(palette = "Set1", name = "Drug") +
  scale_y_continuous(breaks = pretty_breaks(2)) +
  xlab("Spearman correlation between association coefficient and screen p-value") +
  ylab("Count")

ggsave("figure_pdf/fig3a.pdf", width = 7.5, height = 5, units = "cm", device = Cairo::CairoPDF)


corrs %>% select(study, prestwick_ID, is_ppi, corr_pv) %>%
  rename(corr_in_vivo_in_vitro = corr_pv) %>% arrange(study, -corr_in_vivo_in_vitro) %>%
  write_tsv("figure_source_data/3a.tsv")
