#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(tidyverse, stringi, here, ggstance, forcats, gtable, ggrepel, scales)
})

source(here("processing_config.R"))

d <- read_tsv(here("annotations/Falony_Table_S6.txt"))


species_overview <- read_tsv(paste0(output_data_path, "species_overview.tsv"), col_types = cols(
  NT_code = col_character(),
  species_name = col_character(),
  strain = col_integer(),
  specI_cluster = col_character(),
  species = col_character(),
  is_unique = col_logical(),
  species_label = col_character(),
  biosafety = col_character()
))

combined_pv_all <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
)) %>% group_by(prestwick_ID) %>% mutate(any_hit = any(hit)) %>% ungroup()


p2at <- read_tsv(paste0(output_data_path, "prestwick_atc.tsv"))

d <- d %>% filter(!is.na(NT_codes)) %>% rowwise() %>% do(data.frame(., NT_code = stri_split_fixed(.$NT_codes, " ")[[1]], stringsAsFactors = F))
d <- d %>% filter(NT_code %in% combined_pv_all$NT_code)

selectATC <- function(s) {
  if (stri_detect_fixed(s, " ")) {
    unique(unlist(lapply(stri_split_fixed(s, " ")[[1]], selectATC)))
  } else {
    unique(p2at$ATC[ stri_startswith_fixed(p2at$ATC, s) ])
  }
}

d <- d %>% ungroup() %>% group_by(ATC) %>% do({
  a <- .
  x <- selectATC(.$ATC[1])
  if (length(x) == 0) {
    data.frame()
  } else {
    b <- tibble(ATC = .$ATC[1], matchedATC = x)
    inner_join(a, b, by = "ATC") %>% select(-ATC) %>% rename(ATC = matchedATC)
  }
})

d <- d %>% inner_join(p2at %>% select(ATC, prestwick_ID), by="ATC")

combined_pv <- bind_rows(combined_pv_all, data.frame(NT_code = "dummy")) %>% left_join(d, by = c("NT_code", "prestwick_ID"))
combined_pv <- combined_pv %>% mutate(Level = ordered(Level, levels = c("Phylum", "Class", "Order", "Family", "Genus", "Species")))
combined_pv <- combined_pv %>% group_by(prestwick_ID) %>% filter(any(!is.na(Falony_Coefficient))) %>% ungroup()
combined_pv <- combined_pv %>% mutate( Falony_Coefficient_repl = ifelse(is.na(Falony_Coefficient), 0, Falony_Coefficient) )

combined_pv %>% filter(!is.na(Covariate)) %>%
  ggplot(aes(Genera, Falony_Coefficient)) + geom_point() + facet_wrap(~Covariate, scales="free_x")


boxy <- function(v, log=F) {
  if (log) vv <- log(v)
  qs <- quantile(vv, c(0.25, 0.5, 0.75))
  iqr <- qs[3] - qs[1]

  if (iqr == 0) {
    d <- data.frame( bmin = qs[1], bmid = qs[2], bmax = qs[3], lmin = qs[1], lmax = qs[3], n = length(v))
  } else {
    if (log) {
      d <- data.frame( bmin = exp(qs[1]), bmid = exp(qs[2]), bmax = exp(qs[3]), lmin = exp(qs[1] - 1.5*iqr), lmax = exp(qs[3] + 1.5*iqr), n = length(v))
    } else {
      d <- data.frame( bmin = qs[1], bmid = qs[2], bmax = qs[3], lmin = qs[1] - 1.5*iqr, lmax = qs[3] + 1.5*iqr, n = length(v))
    }

    within_range <- v[ which(v > d$lmin & v < d$lmax) ]

    d$lmax <- max(within_range)
    d$lmin <- min(within_range)
  }

  d
}

combined_pv_merged <- combined_pv %>% filter(!is.na(Covariate)) %>%
  mutate(Covariate = factor(Covariate))

ordering <- combined_pv_merged %>% select(Covariate, Genera) %>% unique() %>% group_by(Covariate) %>%
  mutate( i = as.integer(factor(Genera)) ) %>% ungroup() %>%
  arrange(Covariate, i) %>% mutate( y = 1:n() + as.integer(Covariate) )

combined_pv_merged <- combined_pv_merged %>% inner_join(ordering, by = c("Genera", "Covariate")) %>%
  select(y, NT_code, Covariate, Genera, prestwick_ID, target_species, any_hit, pv_comb, pv_comb_fdr_BH, Falony_Coefficient, Falony_FDR) %>% unique() %>%
  arrange(y)

combined_pv_merged <- combined_pv_merged %>% mutate(Genera = stri_replace_all_fixed(Genera, "_", " ") %>% stri_replace_all_fixed("unclassified", "unclass."))

coeffs <- combined_pv_merged %>% select(y, Covariate, Genera, Falony_Coefficient, Falony_FDR) %>% unique()

boxes <- combined_pv_merged %>% group_by(y) %>% filter(!is.na(pv_comb_fdr_BH)) %>% do(boxy(.$pv_comb_fdr_BH, log=T))
outliers <- combined_pv_merged %>% right_join(boxes, by="y") %>% filter(pv_comb_fdr_BH < lmin | pv_comb_fdr_BH > lmax | n < 10)

N <- max(ordering$y) + 1

pb <- ggplot(boxes %>% filter(n >= 10)) +
  geom_hline(data = boxes, aes(yintercept = y), color = "lightgrey", size = 0.25) +
  geom_linerangeh(aes(y = y, xmin = lmin, xmax = bmin)) +
  geom_linerangeh(aes(y = y, xmin = bmax, xmax = lmax)) +
  geom_crossbarh(aes(y = y, xmin = bmin, x = bmid, xmax = bmax), width = 0.5, fill = "white") +
  geom_vline(xintercept = 0.01) +
  geom_point(data = outliers, aes(x = pv_comb_fdr_BH, y = y), size = 0.5) +
  scale_x_log10() +
  scale_y_continuous( limits =c(1.5, N + 0.5), expand = c(0,0)) +
  xlab("q-value of\nanticommensal activity") +
  theme_minimal() + theme(panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(),
                          axis.text.x = element_text(size = 6),  axis.title.x = element_text(size = 7))


X0 <- -0.7
X1 <- 0
X2 <- 1
X3 <- 2
m <- 0.05

labels <- ordering %>% group_by(Covariate) %>% filter(y == max(y))

b <- seq(2, 6, 2)

pt <-
  ggplot(coeffs, aes(Falony_Coefficient, y)) +
  geom_hline(aes(yintercept = y), color="lightgrey", size = 0.25) +
  geom_point(aes(size = -log10(Falony_FDR))) +
  scale_size_area(name = "q-value in cohort study", max_size = 4, breaks = b, labels = 10**-b) +
  scale_color_manual(values = c("#b2df8a", "#1f78b4")) +
  scale_x_continuous(expand = c(0.1, 0)) +
  geom_vline(xintercept = 0) +
  geom_rect(data = labels, aes(xmin = -Inf, xmax = Inf, ymin = y + 0.75, ymax = y + 1.25), fill = "white", size = 0, inherit.aes = F, alpha = 0.75) +
  geom_text(data = labels, aes(x = -Inf, y = y + 1, label = Covariate), hjust = "inward", size = 2) +
  theme_minimal() +
  theme(axis.text.y = element_text(hjust = 0, size = 6), legend.position = "bottom",
        legend.title = element_text(size = 7),
        legend.text = element_text(size = 6),
        legend.box.margin = margin(0),
        legend.margin = margin(-0.2, unit = "line"),
        legend.key.height = unit(0.75, "line"),
        panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(), legend.box = "vertical",
        axis.text.x = element_text(size = 6),  axis.title.x = element_text(size = 7)) +
  scale_y_continuous(limits =c(1.5, N + 0.5), expand = c(0,0), breaks = coeffs$y, labels = coeffs$Genera) +
  xlab("Association coefficient\nin cohort study")

ggplot() + theme_void() + xlim(X0, X3) + ylim(-2, N ) + ggtitle("Flemish cohort study") +
  annotation_custom(gtable_filter(ggplotGrob(pt), "panel"), xmin = X1, xmax = X2-m, ymin = 0.5, ymax = N+0.5) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "axis-l"), xmin = X0, xmax = X1-m, ymin = 0.5, ymax = N+0.5) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "axis-b"), xmin = X1, xmax = X2-m, ymin = 0, ymax = 0.4) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "xlab"), xmin = X1, xmax = X2-m, ymin = -1, ymax = 0) +
  annotation_custom(gtable_filter(ggplotGrob(pt), "guide-box"), xmin = X0, xmax = X3, ymin = -4, ymax = -1) +
  annotation_custom(gtable_filter(ggplotGrob(pb), "panel"), xmin = X2+m, xmax = X3, ymin = 0.5, ymax = N+0.5) +
  annotation_custom(gtable_filter(ggplotGrob(pb), "axis-b"), xmin = X2+m, xmax = X3, ymin = 0, ymax = 0.4) +
  annotation_custom(gtable_filter(ggplotGrob(pb), "xlab"), xmin = X2+m, xmax = X3, ymin = -1, ymax = 0) +
  theme(title = element_text(size = 8))

ggsave("figure_pdf/figED7b.pdf", width = 10, height = 8, units = "cm", device = Cairo::CairoPDF)

cc <- combined_pv_merged %>% group_by(Covariate) %>% filter(length(unique(Genera)) > 1) %>% select(Covariate, target_species, any_hit, NT_code, Falony_Coefficient) %>%
  inner_join(combined_pv_all, by = c("NT_code", "any_hit", "target_species"))

ccc <- cc %>% group_by(Covariate, prestwick_ID, target_species, any_hit) %>% summarise( corr = cor(Falony_Coefficient, pv_comb, method = "s", use = "p"))

rcc <- combined_pv_merged %>% group_by(Covariate) %>% filter(length(unique(Genera)) > 1)  %>% group_by(Covariate) %>% summarise( corr = cor(Falony_Coefficient, pv_comb, method = "s", use = "p"))

ccc <- ccc %>% ungroup() %>% mutate(
  kind = paste(
    recode(target_species, human = "human-targeted drug", "bacteria" = "antibacterial drug"),
    ifelse(any_hit, "with", "without"),
    "anticommensal activity")
) %>% filter(!is.na(corr))




ccc_ <- ccc %>% group_by(Covariate) %>% filter(length(unique(corr)) > 2)
rcc_ <- rcc %>% filter(Covariate %in% ccc_$Covariate)

ggplot(ccc_, aes(corr, fill = kind)) + geom_histogram(binwidth = 0.2, boundary = 0, alpha = 0.5) + facet_wrap(~Covariate, scales = "free_y") +
  geom_vline(data = rcc_, aes(xintercept = corr), size = 0.1) +
  theme_minimal() +
  theme(strip.background = element_rect(fill = "lightgrey", size = 0), panel.grid.minor = element_blank(),
        legend.position = "bottom",
        strip.text = element_text(size = 7),
        axis.text = element_text(size = 6),  axis.title = element_text(size = 7)
  ) + scale_y_continuous(breaks = pretty_breaks(4)) +
  scale_fill_manual(values = c(dbcolors$ab_hit, dbcolors$ht_hit, dbcolors$no_hit), name = "") +
  xlab("Spearman correlation between association coefficient and screen p-value") +
  ylab("Count")

ggsave("figure_pdf/fig3b.pdf", width = 7.5, height = 5, units = "cm", device = Cairo::CairoPDF)

ccc_ %>% select(Covariate, prestwick_ID, kind, corr) %>%
  rename(corr_in_vivo_in_vitro = corr) %>% arrange(Covariate, -corr_in_vivo_in_vitro) %>%
  write_tsv("figure_source_data/3b.tsv")
