#!/usr/bin/env Rscript

if (!require("pacman")) install.packages("pacman")
pacman::p_load(tidyverse, viridis, forcats, stringi, gtable, broom, here)

source(here("processing_config.R"))

threshold <- 1e-4

MICs_ <- read_tsv(paste0(output_data_path, "LM16-150-MIC.tsv"), col_types = cols())

se_candidates <- read_tsv(paste0(output_data_path, "se_candidates.tsv"), col_types = cols()) %>%
  select(chemical_name, prestwick_ID, p.value_all_SE)

se_full_pv <- read_tsv(paste0(output_data_path, "se_all_pv.tsv"), col_types = cols())

prestwick_atc <- read_tsv(paste0(output_data_path, "prestwick_atc.tsv"), col_types = cols())
NSAIDs <- prestwick_atc %>% filter(stri_startswith_fixed(ATC, "M01A") | stri_startswith_fixed(ATC, "N02BA"), !is.na(prestwick_ID))

species_overview <- read_tsv(paste0(output_data_path, "species_overview.tsv"), col_types = cols(
  NT_code = col_character(),
  species_name = col_character(),
  strain = col_integer(),
  specI_cluster = col_character(),
  species = col_character(),
  is_unique = col_logical(),
  species_label = col_character(),
  biosafety = col_character()
))

prestwick_ddd <- read_tsv(paste0(output_data_path, "prestwick_ddd.tsv"), col_types = cols())

## Metformin is for another figure, two of the original candidates became hits in the meantime
MICs <- MICs_ %>% left_join(species_overview %>% select(NT_code, species_label), by="NT_code")
MICs <- MICs %>% inner_join(prestwick_ddd, by = c("prestwick_ID", "chemical_name"))
MICs <- MICs %>% left_join(se_candidates, by=c("prestwick_ID", "chemical_name")) %>% filter(! Drug %in% c("Metformin") , !is.na(p.value_all_SE))

MICs <- MICs %>% mutate(Conc_um = factor(as.integer(Conc * 1000)), chemical_name = fct_reorder(chemical_name, p.value_all_SE),
                        species_label_ = stri_replace_all_regex(species_label, "^(.)[^ ]+", "$1."),
                        species_label_ = fct_reorder(species_label_, Conc, mean))


excr <- read_tsv("annotations/drugbank_elimination_processed.tsv")

MICs <- MICs %>% left_join(excr %>% select(prestwick_ID, colon_conc_uM))

MICs <- MICs %>% mutate(confirmed = ifelse(qualifier == ">", "not active", "reduced bacterial growth"))
MICs <- MICs %>% mutate(dose_ratio = Conc*1e-3/amount_mole, plasma_conc_ratio = Conc*1e-3/mean_conc, colon_conc_ratio = Conc*1000/colon_conc_uM)

MICs_merged <- MICs %>% group_by(chemical_name, prestwick_ID, p.value_all_SE, confirmed) %>%
  summarise(n = n(), dose_ratio = exp(mean(log(dose_ratio)))) %>% arrange(-n)


N <- 1000
conc <- (sort(unique(MICs$Conc)))
cc <- conc - min(conc)
cc <- cc / max(cc)
pal <- viridis(N+1)[N+1-as.integer(N*cc)]
Ndrug <- length(unique(MICs$prestwick_ID))

ph <- ggplot(MICs, aes(chemical_name, as.integer(species_label_), fill=Conc_um)) + geom_raster() +
  scale_fill_manual(values = pal, name = expression(IC[25]*" (µM)")) +
  geom_text(aes(label = qualifier, color = Conc > 1.5), data = MICs %>% filter(qualifier != "="), show.legend = F) +
  scale_color_manual(values = c("black", "white")) + theme_minimal() +
  scale_x_discrete(position = "top") + xlab("") + ylab("") +
  theme(panel.grid = element_blank(), axis.text.x.top = element_text(angle = 45, hjust = 0),
        axis.text.y = element_text(face="italic"))


pd <- ggplot(MICs %>% select(chemical_name, amount_mole) %>% unique(), aes(as.integer(chemical_name), amount_mole)) +
  geom_point(shape=17) + scale_y_log10() + theme_minimal() + ylab("Dose (mol)") +
  scale_x_continuous(limits = c(0.5, Ndrug+0.5), expand = c(0,0), breaks = 1:Ndrug) +
  theme(panel.grid.minor.x = element_blank())

MICs_max <- MICs_merged %>% group_by(chemical_name, prestwick_ID, p.value_all_SE) %>%
  summarise(dose_ratio = max(dose_ratio))

PVLIM <- log10(range(MICs$p.value_all_SE))
PVLIM <- PVLIM + c(-0.5, 0.5)

pconn <- ggplot(MICs_max, aes(x = ((as.integer(chemical_name)-0.5)/Ndrug)*(PVLIM[2]-PVLIM[1])+PVLIM[1], xend = log10(p.value_all_SE), y = 1, yend = 0)) +
  geom_segment(color = "#eeeeee") + theme_void() + scale_x_continuous(limits = PVLIM, expand = c(0,0)) +
  scale_y_continuous(expand=c(0,0))

pr <- ggplot(MICs, aes(p.value_all_SE, dose_ratio, color = confirmed)) +
  geom_vline(xintercept = 1e-4) +
  geom_segment(data = MICs_max, aes(xend = p.value_all_SE, yend = Inf), color = "#eeeeee", show.legend = F) +
  geom_smooth(method = "lm", show.legend = F, fill = "#cccccc") + theme_minimal() +
  geom_point(data = MICs_merged, aes(size=n)) +
  scale_color_manual(values = c("#cccccc", dbcolors$new_hit), name = expression(IC[25]*" outcome"), labels = c("not active", "reduced\nbacterial\ngrowth")) +
  scale_size_area(name = "Number of\nstrains") +
  scale_y_log10(breaks=10**(-1:3)) + scale_x_log10(limits = 10**PVLIM, expand = c(0,0)) +
  xlab("p-value: enrichment of antibiotics-related side effects per drug") +
  ylab("Estimated number of doses\nto reach MIC") +
  # ylab("Ratio between dose and MIC\n(estimated volume of distribution, l)") +
  theme(panel.grid.major.x = element_blank(), panel.grid.minor.x = element_blank())

Dmin <- -10
Rmax <- -12
Rmin <- -35

p <- ph + theme(legend.justification = "top") +
  annotation_custom(gtable_filter(ggplotGrob(pd), "panel"), xmin = 0.5, xmax = Ndrug+0.5, ymin = Dmin, ymax = 0) +
  annotation_custom(gtable_filter(ggplotGrob(pd), "axis-l"), xmin = -0.5, xmax = -0.5, ymin = Dmin, ymax = 0) +
  annotation_custom(gtable_filter(ggplotGrob(pd), "lab-l"), xmin = -2.5, xmax = -2.5, ymin = Dmin, ymax = 0) +
  annotation_custom(gtable_filter(ggplotGrob(pconn), "panel"), xmin = 0.5, xmax = Ndrug+0.5, ymin = Rmax, ymax = Dmin) +
  annotation_custom(gtable_filter(ggplotGrob(pr), "panel"), xmin = 0.5, xmax = Ndrug+0.5, ymin = Rmin, ymax = Rmax) +
  annotation_custom(gtable_filter(ggplotGrob(pr), "axis-l"), xmin = -0.5, xmax = -0.5, ymin = Rmin, ymax = Rmax) +
  annotation_custom(gtable_filter(ggplotGrob(pr), "lab-l"), xmin = -2.5, xmax = -2.5, ymin = Rmin, ymax = Rmax) +
  annotation_custom(gtable_filter(ggplotGrob(pr), "axis-b"), xmin = 0.5, xmax = Ndrug+0.5, ymin = Rmin, ymax = Rmin) +
  annotation_custom(gtable_filter(ggplotGrob(pr), "lab-b"), xmin = 0.5, xmax = Ndrug+0.5, ymin = Rmin-2, ymax = Rmin-2) +
  annotation_custom(gtable_filter(ggplotGrob(pr), "guide-box"), xmin = Ndrug+2, xmax = Ndrug+5, ymin = Rmin, ymax = Rmax) +
  scale_y_continuous(limits = c(Rmin-3, 19), expand=c(0,0), breaks = seq(1, 18), labels = levels(MICs$species_label_))

ggsave(here("figure_pdf/figED8a.pdf"), height = 9, width = 9)

MICs_cumul <- MICs %>% group_by(prestwick_ID, p.value_all_SE, confirmed) %>%
  summarise(n = n()) %>% arrange(-n)

combined_hits <- read_tsv(paste0(output_data_path, "combined_hits.tsv"), col_types = cols())

all_drugs <- se_full_pv %>% full_join(prestwick_ddd %>% select(-chemical_name), by = "prestwick_ID") %>% filter(!is.na(prestwick_ID))

drugs <- MICs %>% group_by(prestwick_ID, Drug, p.value_all_SE) %>% summarise(n_confirmed = sum(qualifier != ">"))
drugs <- drugs %>% left_join(prestwick_ddd, by = "prestwick_ID")

all_drugs <- all_drugs %>% left_join(combined_hits, by = "prestwick_ID")

all_drugs <- all_drugs %>% mutate(hit_ = ifelse(hit, "Anticommensal", "Other drug"))

all_drugs$candidate <- all_drugs$prestwick_ID %in% drugs$prestwick_ID

R <- range(all_drugs$p.value_all_SE, na.rm = T)

all_drugs_t <- all_drugs %>% filter(target_species == "human", !is.na(p.value_all_SE), !is.na(amount_mole), !is.na(hit))
all_drugs_t <- all_drugs_t %>% left_join(MICs %>% group_by(prestwick_ID) %>% summarise(confirmed = any(qualifier != ">")), by="prestwick_ID")
all_drugs_t <- all_drugs_t %>% mutate(
  exthit = ifelse(hit, "screen", ifelse(is.na(confirmed), "not tested", ifelse(confirmed, "MIC", "no")))
) %>% arrange(desc(exthit))

clabel <- data.frame(x = threshold, y = 2e4, label = c("26 candidate drugs  ", "  16 control drugs "))

D <- 20*3

## COUNT FOR PAPER
all_drugs_t %>% filter(!hit) %>% nrow()

p <- ggplot(all_drugs_t %>% filter(!hit), aes(p.value_all_SE, amount_mole*1e6, color=exthit)) +
  geom_vline(xintercept = threshold) +
  geom_hline(yintercept = D, color = "black", linetype = "dashed") +
  geom_text(data = clabel[1,], aes(x=x, y=y, label=label), inherit.aes = F, vjust = "center", hjust="right", size=2) +
  geom_text(data = clabel[2,], aes(x=x, y=y, label=label), inherit.aes = F, vjust = "center", hjust="left", size=2) +
  geom_point(shape = 17, size=1) +
  scale_x_log10(limits = R, breaks = 10**seq(-10,-1,3)) +
  scale_y_log10(expand = c(0,0), limits = c(1e-2, 3e4)) +
  # annotation_custom(gtable_filter(ggplotGrob(plegend), "guide-box"), xmin = -9, xmax = -7, ymin = -1.5, ymax = -2) +
  scale_color_manual(values = c(dbcolors$new_hit, dbcolors$no_hit_alone, "#dddddd"),
                     breaks = c("MIC", "no", "not tested"),
                     labels = c("with anticommensal activity", "inactive", "not tested"),
                     name = "Activity at higher concentrations:") +
  theme_minimal() +
  theme(panel.grid.minor = element_blank(),
        legend.position = "bottom", legend.direction = "vertical",
        legend.title = element_text(size = 7),
        legend.text = element_text(size = 6),
        legend.margin = margin(1),
        legend.key.height = unit(0.75, "line"),
        axis.title = element_text(size = 7),
        axis.text = element_text(size = 6),
        plot.title = element_text(face = "bold", size = 8)
  ) +
  xlab("p-value: enrichment of ABX-related side effects per drug") +
  ylab("Dose in humans (µmol)")


ggsave(here("figure_pdf/figED8b.pdf"), width = 8, height = 10, units="cm")

ggplot(all_drugs_t %>% filter(!hit), aes(p.value_all_SE)) +
  geom_histogram(boundary = threshold, binwidth = 1, fill = dbcolors$no_hit) +
  geom_vline(xintercept = threshold) +
  geom_text(data = clabel[1,], aes(x=x, y=90, label=label), inherit.aes = F, vjust = "center", hjust="right", size=2) +
  geom_text(data = clabel[2,], aes(x=x, y=90, label=label), inherit.aes = F, vjust = "center", hjust="left", size=2) +
  scale_x_log10() +
  # annotation_custom(gtable_filter(ggplotGrob(plegend), "guide-box"), xmin = -9, xmax = -7, ymin = -1.5, ymax = -2) +
  theme_minimal() +
  theme(panel.grid.minor = element_blank(),
        legend.position = "bottom", legend.direction = "vertical",
        legend.title = element_text(size = 7),
        legend.text = element_text(size = 6),
        legend.margin = margin(1),
        legend.key.height = unit(0.75, "line"),
        axis.title = element_text(size = 7),
        axis.text = element_text(size = 6),
        plot.title = element_text(face = "bold", size = 8)
  ) +
  xlab("p-value: enrichment of ABX-related side effects per drug") +
  ylab("Count of drugs")

all_drugs_t %>% select(prestwick_ID, chemical_name, hit, candidate, p.value_all_SE) %>% rename(p.value_SE_enrichment = p.value_all_SE) %>%
  write_tsv("figure_source_data/3d.tsv")

ggsave(here("figure_pdf/fig3d_histogram_pv.pdf"), width = 8, height = 4, units="cm")


getGTT <- function(p.value_all_SE) ifelse(p.value_all_SE >= threshold, "less similar than threshold", "more similar than threshold")
getGTT2 <- function(p.value_all_SE) ifelse(p.value_all_SE <= threshold, "Candidate drugs", "Control drugs")

all_drugs_t <- all_drugs_t %>% mutate(gtT = getGTT(p.value_all_SE), hit_ = ifelse(hit, "anticommensal", "other drug"))

MICs$gtT <- getGTT2(MICs$p.value_all_SE)
drugs$gtT <- getGTT2(drugs$p.value_all_SE)

#### VALUES FOR PAPER
MICs %>% select(prestwick_ID, gtT) %>% unique() %>% group_by(gtT) %>% summarise(n())
drugs %>% group_by(n_confirmed > 0) %>% summarise(n())
drugs %>% group_by(gtT, n_confirmed > 0) %>% summarise(n())
MICs %>% select(NT_code) %>% unique() %>% nrow()

#### PVALUE FOR PAPER
pvC <- MICs %>% group_by(confirmed) %>% do({
  m <- .$gtT == "Candidate drugs"
  wilcox.test(.[m,]$dose_ratio, .[!m,]$dose_ratio) %>% tidy()
})

pvCC <- MICs %>% group_by(confirmed) %>% do({
  m <- .$gtT == "Candidate drugs"
  wilcox.test(.[m,]$colon_conc_ratio, .[!m,]$colon_conc_ratio) %>% tidy()
})


pvD <- all_drugs_t %>% group_by(gtT) %>% do({
  m <- .$hit
  wilcox.test(.[m,]$amount_mole, .[!m,]$amount_mole) %>% tidy()
})


SC <- scale_y_log10(breaks=c(1, 10, 100, 1000), limits = c(2e-2, 1e4))

MICp <- MICs %>% filter(confirmed == "reduced bacterial growth") %>%
  group_by(gtT) %>% mutate(
    gtT_ = paste0(stri_replace_all_fixed(gtT, " ", "\n"), "\nN = ", n()),
    gtT__ = paste0(gtT, "\nN = ", n()))


B <- c(0.1, 1, 10, 100, 1000)

ggplot(MICp, aes(gtT__, dose_ratio)) + geom_boxplot(outlier.size = 0.5) +
  geom_text(data = pvC[2,], aes(x = 1.5, y = Inf, label = sprintf("p = %.2g", p.value)), inherit.aes = F, vjust = "inward", size = 2.5, hjust = "inward") +
  scale_y_log10(breaks=B, labels = sapply(B, as.character), limits = c(1,1.4)*range(MICp$dose_ratio)) +
  scale_fill_manual(values = c("white", alpha("#66c2a5", 0.5)), name = "side-effect similarity to antibiotics") +
  theme_minimal() +
  theme(panel.grid.major.x = element_blank(), legend.position = "bottom", legend.justification = "left", legend.direction = "vertical",
        axis.text.y = element_text(size = 6),
        axis.text.x = element_text(size=6, color="black"),
        axis.title = element_text(size = 7),
        plot.title = element_text(face = "bold", size = 8)

  ) +
  xlab("") +
  ylab(expression("Estimated number of doses to reach "*IC[25])) +
  coord_flip()

MICp %>% ungroup() %>% select(gtT, prestwick_ID, chemical_name, NT_code, qualifier, Conc, amount_mole, dose_ratio) %>% arrange(gtT, prestwick_ID) %>%
  rename(kind = gtT) %>% write_tsv("figure_source_data/3e.tsv")

ggsave("figure_pdf/fig3e.pdf", width = 8, height = 3, unit = "cm")


MICpc <- MICs %>% filter(confirmed == "reduced bacterial growth", !is.na(colon_conc_ratio)) %>%
  group_by(gtT) %>% mutate(gtT_ = paste0(stri_replace_all_fixed(gtT, " ", "\n"), "\nN = ", n()))

B <- c(0.1, 1, 10, 100, 1000)

ggplot(MICpc, aes(gtT_, colon_conc_ratio)) + geom_boxplot(outlier.size = 0.5) +
  geom_text(data = pvCC[2,], aes(x = 1.5, y = Inf, label = sprintf("p = %.2g", p.value)), inherit.aes = F, vjust = "inward", size = 2.5) +
  scale_y_log10(breaks=B, labels = sapply(B, as.character)) +
  scale_fill_manual(values = c("white", alpha("#66c2a5", 0.5)), name = "side-effect similarity to antibiotics") +
  theme_minimal() +
  theme(panel.grid.major.x = element_blank(), legend.position = "bottom", legend.justification = "left", legend.direction = "vertical",
        axis.text.y = element_text(size = 6),
        axis.text.x = element_text(size=6, color="black"),
        axis.title = element_text(size = 7),
        plot.title = element_text(face = "bold", size = 8)

  ) +
  xlab("") +   ylab(expression("Ratio between "*IC[25]*" and estimated colon concentration"))

ggsave("figure_pdf/figED8e.pdf", width = 4, height = 10, unit = "cm")


### VALUE FOR PAPER
MICs %>% group_by(confirmed, gtT) %>% summarise(fraction_le2 = sum(dose_ratio <= 2) / n(), dose_ratio = median(dose_ratio))
