#!/usr/bin/env Rscript

if (!require("pacman")) install.packages("pacman")
pacman::p_load(tidyverse, viridis, stringr, forcats, stringi, here)

source(here("processing_config.R"))

combined_pv <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))

combined_pv$veterinary <- str_detect(combined_pv$target_species, "^veterinary:")
combined_pv$human_use <- !str_detect(combined_pv$target_species, "^not a drug") & !combined_pv$veterinary

p2d <- read_tsv(paste0(output_data_path, "prestwick_ddd.tsv"), col_types = cols(
  prestwick_ID = col_character(),
  chemical_name = col_character(),
  is_drug = col_logical(),
  is_oral_drug = col_logical(),
  amount_mole = col_double()
))

p2a1 <- read_tsv(paste0(output_data_path, "p2a1.tsv"), col_types = cols(
  prestwick_ID = col_character(),
  target_species = col_character()
))

atc_tree <- read_tsv(paste0(output_data_path, "atc_tree"), col_names = c("ATC", "ATC_name"), col_types = cols(
  ATC = col_character(),
  ATC_name = col_character()
))

p2at <- read_tsv(paste0(output_data_path, "p2at.tsv"), col_types = cols(
  prestwick_ID = col_character(),
  chemical_name = col_character(),
  chemical_formula = col_character(),
  molecular_weight = col_double(),
  cas_number = col_character(),
  therapeutic_class = col_character(),
  therapeutic_effect = col_character(),
  lc_chemical_name = col_character(),
  CID = col_character(),
  CID_flat = col_character(),
  ATC = col_character(),
  target_species = col_character()
))

p2at$veterinary <- str_detect(p2at$target_species, "^veterinary:")
p2at$human_use <- !str_detect(p2at$target_species, "^not a drug") & !p2at$veterinary

chemical_names <- p2at %>% select(prestwick_ID, chemical_name, target_species) %>% unique()
chemical_names_atc <- p2at %>% select(prestwick_ID, chemical_name, target_species, ATC) %>% unique()

cids <- p2at %>% select(prestwick_ID, CID_flat) %>% unique()

# collapse comounds with identical CID
combined_hits <- combined_pv %>%
  filter(human_use, target_species == "human") %>%
  group_by(prestwick_ID) %>% summarise(n_hit = sum(hit)) %>%
  left_join(cids, by="prestwick_ID") %>% group_by(CID_flat) %>%
  summarise(n_hit = max(n_hit), hit = n_hit > 0, prestwick_ID = prestwick_ID[1])


getPV <- function(combined_hits, shuffle = F) {

  if (shuffle) combined_hits$hit <- sample(combined_hits$hit)

  combined_hits_atc <- combined_hits %>% left_join(chemical_names_atc, by="prestwick_ID")

  combined_hits_atc <- combined_hits_atc %>% mutate(
    ATC1 = str_sub(ATC, 1, 1),
    ATC2 = str_sub(ATC, 1, 3),
    ATC3 = str_sub(ATC, 1, 4),
    ATC4 = str_sub(ATC, 1, 5)
  )

  combined_hits_atc0 <- combined_hits_atc %>%
    summarise(n = length(unique(prestwick_ID)), n_hit = length(unique(prestwick_ID[hit])), ATC = "all", parent = "none")

  total_n_hit <- combined_hits_atc0$n_hit
  total_n_non_hit <- combined_hits_atc0$n - total_n_hit

  combined_hits_atc1 <- combined_hits_atc %>% group_by(ATC1) %>%
    summarise(n = length(unique(prestwick_ID)), n_hit = length(unique(prestwick_ID[hit])), parent = "all") %>%
    rename(ATC = ATC1)

  combined_hits_atc2 <- combined_hits_atc %>% group_by(ATC1, ATC2) %>%
    summarise(n = length(unique(prestwick_ID)), n_hit = length(unique(prestwick_ID[hit]))) %>%
    rename(ATC = ATC2, parent = ATC1)

  combined_hits_atc3 <- combined_hits_atc %>% group_by(ATC2, ATC3) %>%
    summarise(n = length(unique(prestwick_ID)), n_hit = length(unique(prestwick_ID[hit]))) %>%
    rename(ATC = ATC3, parent = ATC2)

  combined_hits_atc4 <- combined_hits_atc %>% group_by(ATC3, ATC4) %>%
    summarise(n = length(unique(prestwick_ID)), n_hit = length(unique(prestwick_ID[hit]))) %>%
    rename(ATC = ATC4, parent = ATC3)

  combined_hits_atc_tree <- bind_rows(combined_hits_atc0, combined_hits_atc1, combined_hits_atc2, combined_hits_atc3, combined_hits_atc4) %>%
    arrange(ATC) %>% unique() %>%
    left_join(atc_tree, by="ATC")

  combined_hits_atc_tree %>% inner_join(combined_hits_atc_tree %>% select(ATC, n, n_hit), by = c(parent = "ATC"), suffix = c("_current", "_parent")) %>%
    mutate( n_non_hit_current = n_current - n_hit_current,
            n_hit_non_current = n_hit_parent - n_hit_current,
            n_non_hit_non_current = n_parent - n_hit_non_current - n_non_hit_current - n_hit_current) %>%
    rowwise() %>%
    do({
      pv_tree <- fisher.test( matrix( c(.$n_hit_current, .$n_non_hit_current,
                                        .$n_hit_non_current, .$n_non_hit_non_current), nrow = 2 ), alternative = "g" )$p.value
      pv_overall <- fisher.test( matrix( c(.$n_hit_current, .$n_non_hit_current,
                                           total_n_hit - .$n_hit_current, total_n_non_hit - .$n_non_hit_current), nrow = 2 ), alternative = "g" )$p.value
      data.frame(pv_tree, pv_overall, ., stringsAsFactors = F)
    })
}

combined_hits_atc_tree_pv <- getPV(combined_hits) %>% arrange(pv_overall)

combined_hits_atc_tree_pv <- combined_hits_atc_tree_pv %>% mutate(sl = str_length(ATC)) %>% group_by(sl) %>%
  mutate( adj_pv_overall = p.adjust(pv_overall, method = "BH"),
          adj_pv_tree = p.adjust(pv_tree, method = "BH")) %>%
  ungroup() %>% arrange(adj_pv_overall) %>% select(-sl)

combined_hits_atc0 <- combined_hits %>% left_join(chemical_names_atc, by="prestwick_ID") %>%
  summarise(n = length(unique(prestwick_ID)), n_hit = length(unique(prestwick_ID[hit])), ATC = "all", parent = "none")

total_n_hit <- combined_hits_atc0$n_hit
total_n_non_hit <- combined_hits_atc0$n - total_n_hit

combined_hits_atc_tree_pv %>% mutate(n_hit_non_current = total_n_hit - n_hit_current, n_non_hit_non_current = total_n_non_hit - n_non_hit_current) %>%
  select(ATC, ATC_name, adj_pv_overall, n_hit_current, n_non_hit_current, n_hit_non_current, n_non_hit_non_current) %>%
  write_tsv("figure_source_data/4a.tsv")

pv_cutoff <- 0.1

tree_to_plot <- combined_hits_atc_tree_pv %>%
  filter(str_length(ATC) == 1 | adj_pv_overall < pv_cutoff, stri_length(ATC) <= 4) %>% arrange(ATC)

# add parents until fixpoint
len <- 0
while (nrow(tree_to_plot) != len) {
  len <- nrow(tree_to_plot)
  tree_to_plot <- bind_rows(combined_hits_atc_tree_pv %>% filter(ATC %in% tree_to_plot$parent), tree_to_plot) %>% unique()
}

tree_to_plot$fraction <- tree_to_plot$n_hit_current / tree_to_plot$n_current

tree_to_plot <- tree_to_plot %>% mutate(ATC_label = paste0(ATC, " ", stri_replace_all_regex(str_to_title(ATC_name), ",.*", ""),
                                                           " (", n_hit_current, "/", n_current, ")"))

tree_to_plot <- tree_to_plot %>% arrange(ATC)

tree_to_plot$i <- 1:nrow(tree_to_plot)

tree_to_plot$adj_pv_overall_trunc <- pmax(tree_to_plot$adj_pv_overall, min(tree_to_plot[ tree_to_plot$adj_pv_overall > 0,]$adj_pv_overall) )

tree_to_plot$bg <- ifelse(as.integer(factor(str_sub(tree_to_plot$ATC, 1, 1))) %% 2, "white", "grey")

combined_hits_atc <- combined_hits %>% left_join(chemical_names_atc, by="prestwick_ID")
combined_hits_atc <- combined_hits_atc %>% mutate(
  ATC1 = str_sub(ATC, 1, 1),
  ATC2 = str_sub(ATC, 1, 3),
  ATC3 = str_sub(ATC, 1, 4),
  ATC4 = str_sub(ATC, 1, 5)
)

prestwick_by_atc <- bind_rows(
  combined_hits_atc %>% filter(hit, ATC1 %in% tree_to_plot$ATC) %>% select(prestwick_ID, ATC1) %>% rename(ATC = ATC1),
  combined_hits_atc %>% filter(hit, ATC2 %in% tree_to_plot$ATC) %>% select(prestwick_ID, ATC2) %>% rename(ATC = ATC2),
  combined_hits_atc %>% filter(hit, ATC3 %in% tree_to_plot$ATC) %>% select(prestwick_ID, ATC3) %>% rename(ATC = ATC3),
  combined_hits_atc %>% filter(hit, ATC4 %in% tree_to_plot$ATC) %>% select(prestwick_ID, ATC4) %>% rename(ATC = ATC4)
) %>% group_by(prestwick_ID) %>% mutate(sl = str_length(ATC)) %>% filter(sl == max(sl)) %>% unique() %>%
  select(-sl) %>% ungroup()

isPrefix <- function(a, b) {
  n_a <- str_length(a)
  n_b <- str_length(b)

  ifelse(n_a > n_b, str_sub(a, 1, n_b) == b, str_sub(b, 1, n_a) == a)
}

tree_to_plot_subset <- tree_to_plot %>% filter(adj_pv_overall < pv_cutoff) %>% select(ATC, i, fraction)

shared_hit_ATC <- prestwick_by_atc %>% inner_join(prestwick_by_atc, by = "prestwick_ID") %>% filter(ATC.x < ATC.y, !isPrefix(ATC.x, ATC.y)) %>%
  group_by(ATC.x, ATC.y) %>% summarise(shared = n())

shared_hit_ATC <- shared_hit_ATC %>% inner_join(tree_to_plot_subset, by = c(ATC.x = "ATC")) %>%
  inner_join(tree_to_plot_subset, by = c(ATC.y = "ATC"))

w <- 0.4
X <- 5

tree_to_plot2 <- tree_to_plot %>% mutate(ATC1 = stri_sub(ATC, 1, 1)) %>% group_by(ATC1) %>% mutate(ATC1_fraction = min(fraction))

tree_to_plot2 <- tree_to_plot2 %>% ungroup() %>% arrange(ATC1_fraction, fraction) %>% mutate(i = 1:n())
tree_to_plot2$bg <- ifelse(as.integer(fct_inorder(tree_to_plot2$ATC1)) %% 2, "white", "grey")

tree_to_plot2 <- tree_to_plot2 %>% rowwise() %>%
  mutate(level = recode(stri_length(ATC), `1` = 1, `3` = 2, `4` = 3),
         prefix = ifelse(level == 1, "", paste0(paste(rep("    ", level - 2), collapse = ""), "\U2514 ")),
         ATC_label = paste0(prefix, stri_replace_all_regex(str_to_title(ATC_name), ",.*", ""), " (", n_hit_current, "/", n_current, ")"))

CAIRO_FUDGE <- -0.5

example_number <- tree_to_plot2 %>% ungroup() %>% filter(ATC %in% c("C08", "N05A", "L01B")) %>% select(i) %>% mutate(label = row_number())

ggplot(tree_to_plot2, aes(xmax = 0, xmin = fraction, ymin = i - w, ymax = i + w, fill = adj_pv_overall_trunc)) +
  geom_rect(aes(xmin = 1.01, xmax = X, ymin = -Inf, ymax = Inf), fill="white") +
  geom_rect(aes(xmin = 0, xmax = X, ymin = i - 0.5, ymax = i + 0.5), fill="grey", alpha=0.25, data = tree_to_plot2 %>% filter(bg == "grey")) +
  geom_text(aes(x = 1, y = i, label = ATC_label), hjust = 0, nudge_x = 0.05, size = 2) +
  geom_rect() +
  geom_text(aes(x = 0, y = i, label = ATC, color = adj_pv_overall_trunc > 1e-4), hjust = 0, nudge_x = 0.01, show.legend = F, size = 2) +
  geom_point(data = example_number, aes(x = -0.12, y = i), inherit.aes = F, size = 2, shape=1) +
  geom_text(data = example_number, aes(x = -0.12, y = i, label = label), inherit.aes = F, size = 1.5) +
  scale_color_manual(values = c("black", "white")) +
  scale_fill_viridis(trans="log10", begin = 1, end = 0, name = "p-value: hits are overrepresented") +
  scale_x_continuous(limits = c(-0.22, X), breaks = seq(0, 1, 0.5), expand = c(0,0)) +
  scale_y_reverse(expand = c(0,0)) +
  # scale_size_continuous(range=c(0,2), limits = c(0, max(chem_sim$shared)), breaks = c(1, 5, 10), name = "number compounds: chemically similar (grey) or multiple indications (black)") +
  theme_minimal() + ylab("Indication area") + xlab("Fraction of drugs with anticommensal activity") +
  theme(axis.text.y = element_blank(), panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(),
        axis.text.x = element_text(size = 7+CAIRO_FUDGE),
        axis.title.y = element_text(size = 7+CAIRO_FUDGE),
        axis.title.x = element_text(hjust=0, size = 6+CAIRO_FUDGE),
        legend.position = "bottom",
        legend.title = element_text(size = 7+CAIRO_FUDGE),
        legend.text =  element_text(size = 6+CAIRO_FUDGE),
        legend.key.height = unit(0.75, "line"),
        legend.margin = margin(-1, unit = "line")
        )

ggsave("figure_pdf/fig4a_indications.pdf", width = 8, height = 6, device = Cairo::CairoPDF, units = "cm", pointsize = 10)

#### SUPPL FIG

tree_to_plot <- combined_hits_atc_tree_pv %>% filter(str_length(ATC) == 1 | n_hit_current/n_current >= 0.5 & n_hit_current >= 2) %>% arrange(ATC)

# add parents until fixpoint
len <- 0
while (nrow(tree_to_plot) != len) {
  len <- nrow(tree_to_plot)
  tree_to_plot <- bind_rows(combined_hits_atc_tree_pv %>% filter(ATC %in% tree_to_plot$parent), tree_to_plot) %>% unique()
}

tree_to_plot$fraction <- tree_to_plot$n_hit_current / tree_to_plot$n_current

tree_to_plot <- tree_to_plot %>% mutate(ATC_label = paste0(ATC, " ", stri_replace_all_regex(str_to_title(ATC_name), ",.*", ""),
                                                           " (", n_hit_current, "/", n_current, ")"))

tree_to_plot <- tree_to_plot %>% arrange(ATC)

tree_to_plot$i <- 1:nrow(tree_to_plot)

tree_to_plot$adj_pv_overall_trunc <- pmax(tree_to_plot$adj_pv_overall, min(tree_to_plot[ tree_to_plot$adj_pv_overall > 0,]$adj_pv_overall) )

tree_to_plot$bg <- ifelse(as.integer(factor(str_sub(tree_to_plot$ATC, 1, 1))) %% 2, "white", "grey")

combined_hits_atc <- combined_hits %>% left_join(chemical_names_atc, by="prestwick_ID")
combined_hits_atc <- combined_hits_atc %>% mutate(
  ATC1 = str_sub(ATC, 1, 1),
  ATC2 = str_sub(ATC, 1, 3),
  ATC3 = str_sub(ATC, 1, 4),
  ATC4 = str_sub(ATC, 1, 5)
)

prestwick_by_atc <- bind_rows(
  combined_hits_atc %>% filter(hit, ATC1 %in% tree_to_plot$ATC) %>% select(prestwick_ID, ATC1) %>% rename(ATC = ATC1),
  combined_hits_atc %>% filter(hit, ATC2 %in% tree_to_plot$ATC) %>% select(prestwick_ID, ATC2) %>% rename(ATC = ATC2),
  combined_hits_atc %>% filter(hit, ATC3 %in% tree_to_plot$ATC) %>% select(prestwick_ID, ATC3) %>% rename(ATC = ATC3),
  combined_hits_atc %>% filter(hit, ATC4 %in% tree_to_plot$ATC) %>% select(prestwick_ID, ATC4) %>% rename(ATC = ATC4)
) %>% group_by(prestwick_ID) %>% mutate(sl = str_length(ATC)) %>% filter(sl == max(sl)) %>% unique() %>%
  select(-sl) %>% ungroup()

tree_to_plot_subset <- tree_to_plot %>% filter(adj_pv_overall < pv_cutoff) %>% select(ATC, i, fraction)

shared_hit_ATC <- prestwick_by_atc %>% inner_join(prestwick_by_atc, by = "prestwick_ID") %>% filter(ATC.x < ATC.y, !isPrefix(ATC.x, ATC.y)) %>%
  group_by(ATC.x, ATC.y) %>% summarise(shared = n())

shared_hit_ATC <- shared_hit_ATC %>% inner_join(tree_to_plot_subset, by = c(ATC.x = "ATC")) %>%
  inner_join(tree_to_plot_subset, by = c(ATC.y = "ATC"))

w <- 0.4
X <- 5

tree_to_plot2 <- tree_to_plot %>% mutate(ATC1 = stri_sub(ATC, 1, 1)) %>% group_by(ATC1) %>% mutate(ATC1_fraction = min(fraction))

tree_to_plot2 <- tree_to_plot2 %>% ungroup() %>% mutate(i = 1:n())
tree_to_plot2$bg <- ifelse(as.integer(fct_inorder(tree_to_plot2$ATC1)) %% 2, "white", "grey")

tree_to_plot2 <- tree_to_plot2 %>% rowwise() %>%
  mutate(level = recode(stri_length(ATC), `1` = 1, `3` = 2, `4` = 3, `5` = 4),
         prefix = ifelse(level == 1, "", paste0(paste(rep("    ", level - 2), collapse = ""), "\U2514 ")),
         ATC_label = paste0(prefix, stri_replace_all_regex(str_to_title(ATC_name), ",.*", ""), " (", n_hit_current, "/", n_current, ")"))

CAIRO_FUDGE <- -0.5

ggplot(tree_to_plot2, aes(xmax = 0, xmin = fraction, ymin = i - w, ymax = i + w, fill = adj_pv_overall_trunc)) +
  geom_rect(aes(xmin = 1.01, xmax = X, ymin = -Inf, ymax = Inf), fill="white") +
  geom_rect(aes(xmin = 0, xmax = X, ymin = i - 0.5, ymax = i + 0.5), fill="grey", alpha=0.25, data = tree_to_plot2 %>% filter(bg == "grey")) +
  geom_text(aes(x = 1, y = i, label = ATC_label), hjust = 0, nudge_x = 0.05, size = 2) +
  geom_rect() +
  geom_text(aes(x = 0, y = i, label = ATC, color = adj_pv_overall_trunc > 1e-4), hjust = 0, nudge_x = 0.01, show.legend = F, size = 2) +
  scale_color_manual(values = c("black", "white")) +
  scale_fill_viridis(trans="log10", begin = 1, end = 0, name = "p-value: hits are overrepresented") +
  scale_x_continuous(limits = c(0, X), breaks = seq(0, 1, 0.5), expand = c(0,0)) +
  scale_y_reverse(expand = c(0,0)) +
  # scale_size_continuous(range=c(0,2), limits = c(0, max(chem_sim$shared)), breaks = c(1, 5, 10), name = "number compounds: chemically similar (grey) or multiple indications (black)") +
  theme_minimal() + ylab("Indication area") + xlab("Fraction of drugs with anticommensal activity") +
  theme(axis.text.y = element_blank(), panel.grid.major.y = element_blank(), panel.grid.minor.y = element_blank(),
        axis.text.x = element_text(size = 7+CAIRO_FUDGE),
        axis.title.y = element_text(size = 7+CAIRO_FUDGE),
        axis.title.x = element_text(hjust=0, size = 6+CAIRO_FUDGE),
        legend.position = "bottom",
        legend.title = element_text(size = 7+CAIRO_FUDGE),
        legend.text =  element_text(size = 6+CAIRO_FUDGE),
        legend.key.height = unit(0.75, "line"),
        legend.margin = margin(-1, unit = "line")
  )

ggsave("figure_pdf/figED9.pdf", width = 8.27, height = 11.69, device = Cairo::CairoPDF, pointsize = 10)
