#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(broom, stringr, readr, tibble, dplyr, ggplot2, gplots, viridis, tidyr,
                 scales, reshape2, forcats, here, purrr, gtable, stringi, ggstance)
})

source(here("processing_config.R"))

combined_pv <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols())

combined_hits <- combined_pv %>% group_by(target_species, prestwick_ID) %>% summarise(hit = any(hit))

atc_tree <- read_tsv(paste0(output_data_path, "atc_tree"), col_names = c("ATC", "ATC_name"), col_types = cols())

p2a <- read_tsv(paste0(output_data_path, "prestwick_atc.tsv"), col_types = cols())

species_overview <- read_tsv(paste0(output_data_path, "species_overview.tsv"), col_types = cols()) %>% filter(!is.na(NT_code))

suppressWarnings(species_annotations <- read_tsv(here("annotations/species_annotations.tsv"), col_types = cols(
  .default = col_character(),
  `Risk group` = col_integer()
)))

species_overview <- species_overview %>% left_join(
  species_annotations %>% select(NT_code, Phylum, Class, Order, Family, Genus, Gram_stain),
  by="NT_code")

combined_pv <- combined_pv %>% left_join(p2a %>% select(prestwick_ID, chemical_name) %>% unique(), by="prestwick_ID")

combined_pv <- combined_pv %>% left_join(species_overview %>% select(NT_code, species_label, Phylum, Class, Order, Family, Genus, Gram_stain), by="NT_code")
combined_pv <- combined_pv %>% group_by(prestwick_ID) %>% mutate(n_hit = sum(hit), any_hit = n_hit > 0)

species_overview <- species_overview %>% filter(NT_code %in% combined_pv$NT_code)

filtered_pv <- as_tibble(rbind(
  data.frame(combined_pv, filtered = "all compounds"),
  data.frame(combined_pv %>% filter(any_hit, target_species == "human"), filtered = "hits among human-targeted drugs"),
  data.frame(combined_pv %>% filter(any_hit, target_species == "bacteria"), filtered = "hits among anti-bacterials"),
  data.frame(combined_pv %>% filter(any_hit, human_use, target_species != "human", target_species != "bacteria"), filtered = "hits among other human-use drugs")
))


s <- "hits among human-targeted drugs"

m <- filtered_pv %>% filter(filtered == s) %>% mutate(log_pv_comb = log10(pv_comb)) %>%
  acast(species_label~prestwick_ID, value.var="log_pv_comb") %>% cor(use="p")

tanimoto_matrix <- readRDS(paste0(output_data_path, "tanimoto_matrix.rds"))
tanimoto_matrix <- tanimoto_matrix[ rownames(m), colnames(m) ]
diag(tanimoto_matrix) <- 1

tanimoto_matrix[ upper.tri(tanimoto_matrix) ] <- t(tanimoto_matrix)[ upper.tri(tanimoto_matrix)  ]

cl <- hclust(as.dist(1-tanimoto_matrix))
# cl <- hclust(dist(m))
compound_order <- cl$labels[ rev(cl$order) ]

common_prefix1 <- function(l1, l2) {
  n <- min(length(l1), length(l2))
  sum(cumprod(l1[1:n] == l2[1:n]))
}

common_prefix <- function(a, b) {
  a <- str_split(a, "")
  b <- str_split(b, "")
  map2(a, b, common_prefix1) %>% unlist()
}

p2a_hit <- p2a %>% filter(prestwick_ID %in% compound_order) %>% select(prestwick_ID, chemical_name, ATC)
p2a_comb <- merge(p2a_hit, p2a_hit, by=c(), suffixes = c("1", "2")) %>% as_data_frame()
# N01AX07
# 1234567
# 1123445
p2a_comb <- p2a_comb %>% mutate(ATC_overlap = common_prefix(ATC1, ATC2),
                    ATC_overlap = recode(ATC_overlap, `1` = 1, `2` = 1, `3` = 2, `4` = 3, `5` = 4, `6` = 4, `7` = 5))

p2a_comb <- p2a_comb %>% mutate(common_ATC = stri_sub(ATC1, 1, recode(ATC_overlap, `1` = 1, `2` = 3, `3` = 4, `4` = 5, `5` = 7)))

p2a_comb <- p2a_comb %>% group_by(prestwick_ID1, prestwick_ID2, chemical_name1, chemical_name2) %>%
  filter(ATC_overlap == max(ATC_overlap)) %>% filter(row_number() == 1)

dt <- tanimoto_matrix[compound_order, compound_order] %>% melt(c("prestwick_ID1", "prestwick_ID2"), as.is = T, value.name = "tanimoto") %>% as_data_frame()

d <- m[compound_order, compound_order] %>% melt(c("prestwick_ID1", "prestwick_ID2"), as.is = T, value.name = "corr")
d <- d %>% as_data_frame() %>% left_join(p2a_comb, by = c("prestwick_ID1", "prestwick_ID2")) %>%
  mutate(chemical_name1 = as.ordered(fct_inorder(chemical_name1)), chemical_name2 = as.ordered(fct_inorder(chemical_name2)))

d <- d %>% left_join(dt, by = c("prestwick_ID1", "prestwick_ID2"))

p2a_kind <- p2a %>% select(prestwick_ID, ATC) %>% mutate(
  ATC2 = stri_sub(ATC, 1, 3),
  ATC3 = stri_sub(ATC, 1, 4),
  ATC4 = stri_sub(ATC, 1, 5),
  kind = recode(ATC2,
                C08 = "Calcium channel blockers",
                .default = ATC3),
  kind = recode(kind,
                L01B = "Antimetabolites",
                N05A = "Antipsychotics",
                `Calcium channel blockers` = "Calcium channel blockers",
                .default = "other"),
  #               .default = ATC4),
  # kind = recode(kind,
  #               A02BC = "Proton-pump inhibitors",
  #               Antimetabolites = "Antimetabolites",
  #               Antipsychotics = "Antipsychotics",
  #               `Calcium channel blockers` = "Calcium channel blockers",
  #               .default = "other"),
  # kind = factor(kind, levels = c("Antipsychotics", "Antimetabolites", "Calcium channel blockers", "Proton-pump inhibitors", "other"))
  kind = factor(kind, levels = c("Antipsychotics", "Antimetabolites", "Calcium channel blockers", "other"))
)

table(p2a_kind$kind)

p2a_kind <- p2a_kind %>% filter(!is.na(prestwick_ID)) %>% select(prestwick_ID, kind) %>% unique() %>% group_by(prestwick_ID) %>%
  filter(n() == 1 | kind != "other")

compounds <- p2a %>% filter(prestwick_ID %in% compound_order) %>% group_by(prestwick_ID, chemical_name) %>%
  summarise(ATC = ifelse(n() > 2, paste(c(ATC[1:2], "..."), collapse = ", "), paste(ATC, collapse = ", "))) %>% ungroup() %>%
  left_join(p2a_kind, by = "prestwick_ID") %>%
  left_join(combined_pv %>% filter(hit) %>% group_by(prestwick_ID) %>% count(), by = "prestwick_ID") %>%
  mutate(prestwick_ID = ordered(prestwick_ID, levels = compound_order)) %>% arrange(prestwick_ID) %>%
  mutate(label = sprintf("%2d %s", n, ATC))

compounds$label <- stri_replace_all_regex(compounds$label, "^ ", "  ")

N <- length(compound_order)
L <- c(0.5, (0.5+N))
LX <- c(0.5, (0.5+N+30))

d1 <- d %>% filter(chemical_name1 < chemical_name2)
d2 <- d %>% filter(chemical_name1 > chemical_name2)

cor.test(d1$corr, d1$tanimoto, method="s")$p.value

p <- ggplot(d1, aes(tanimoto, corr)) + geom_point(size = 0.1) +
  theme_minimal() + geom_density_2d(color="red", size = 0.1) +
  xlab("2D chemical similarity") +
  ylab("Spearman correlation between activity profiles") +
  geom_smooth(method="lm", size = 0.5) +
  theme(axis.text = element_text(size = 6),  axis.title = element_text(size = 7),
        panel.grid.minor = element_blank()
  )

ggsave("figure_pdf/figED10c.tiff", p, width = 5, height = 7, units = "cm", dpi = 300)

d_atc <- bind_rows(
  # d1 %>% filter(common_ATC %in% c("N", "N05", "N05A", "C", "C08", "L", "L01", "L01B")) %>% mutate(selection = common_ATC),
  d1 %>% filter(common_ATC %in% c("N05A", "C08", "L01B")) %>% mutate(selection = common_ATC),
  d1 %>% filter(!(common_ATC %in% c("N05A", "C08", "L01B"))) %>% mutate(selection = "other"),
  d1 %>% mutate(selection = "all")
) %>% filter(ATC_overlap < 5, ATC_overlap > 1 & ATC_overlap < 4 | selection != "other")

d_atc <- d_atc %>% mutate(selection = fct_rev(selection))

labelT <- "2D chemical similarity\n(Tanimoto score)"
labelC <- "Spearman correlation\nbetween activity profiles"

d_atc2 <- d_atc %>% gather(measure, value, tanimoto, corr) %>%
  mutate(measure = recode(measure, tanimoto = labelT, corr = labelC))

pv <- d_atc2 %>% group_by(ATC_overlap, measure) %>% filter(any(selection == "other")) %>% do({
  d <- .
  v <- (d %>% filter(selection == "other"))$value
  d %>% filter(!(selection %in% c("other", "all"))) %>% group_by(selection) %>% do({
    if (nrow(.) == 0) return(data.frame())
    wilcox.test(.$value, v) %>% tidy()
  })
}) %>% arrange(selection)

pv <- pv %>% mutate(label = sprintf("p = %.1g", p.value))

pv <- pv %>% group_by(ATC_overlap, measure) %>% mutate(y = row_number())

d_atc2 <- d_atc2 %>% mutate(ATC_overlap_name = recode(ATC_overlap,
  `0` = "all unrelated compounds",
  `1` = "same main anatomical group",
  `2` = "same main therapeutic group",
  `3` = "same pharmacological group",
  `4` = "same chemical group")) %>% ungroup()

d_atc2 <- d_atc2 %>% mutate(selection_ = recode(selection,
  all = "all",
  C08 = "Calcium channel blockers (C08)",
  L01B = "Antimetabolites (L01B)",
  N05A = "Antipsychotics (N05A)",
  other = "other compound pairs"),
  selection_ = ifelse(selection_ == "all", ATC_overlap_name, as.character(selection_))) %>%
  group_by(ATC_overlap, selection) %>%
  mutate(selection_ = paste0(selection_, " (N = ",n(), ")")) %>%
  ungroup() %>% arrange(selection) %>%
  mutate(selection_ = stri_replace_all_fixed(selection_, ") (", ", ")) %>%
  mutate(selection_ = fct_inorder(selection_))

mylimits <- tribble(
  ~measure, ~value,
  labelT, 1.5,
  labelC, 1.7
)

ggplot(d_atc2, aes(value, selection_)) +
  geom_boxploth(outlier.size = 0.25) +
  facet_grid(ATC_overlap~measure, scales="free", space="free_y") +
  ylab("Subgroup of compounds by number of shared levels in ATC") +
  geom_blank(data = mylimits, aes(x=value), inherit.aes = F) +
  scale_x_continuous(breaks = seq(-1, 1, 0.5)) +
  geom_text(data = pv, aes(x = 1.15-y/20, y = y+0.5, label = label), hjust="left", size=2.5) +
  geom_segment(data = pv, aes(x = 1.1-y/20, xend = 1.1-y/20, y = 1, yend = y+1)) +
  theme_minimal() +
  theme(axis.text = element_text(size = 6),  axis.title = element_text(size = 7),
        strip.text = element_text(size = 7), panel.grid.minor = element_blank()
        )

# ggsave(here("figure_pdf/fig4_boxplots.pdf"), width = 15, height = 10, units = "cm")


dhalf <- d %>% filter(chemical_name1 > chemical_name2) %>%
  mutate(tanimoto_bin = gtools::quantcut(tanimoto, 400))

sampleCorr <- function(prefix) {
  dhalf %>% group_by(tanimoto_bin) %>% select(tanimoto_bin, tanimoto, corr, common_ATC) %>% do({
    x <- .
    n <- sum(stri_startswith_fixed(x$common_ATC, prefix))
    x %>% filter(!stri_startswith_fixed(common_ATC, prefix)) %>% sample_n(n)
  }) %>% ungroup() %>%
  select(-tanimoto_bin)
}

avgSampleCorr <- function(prefix) {
  l <- replicate(10, sampleCorr(prefix))
  tibble(
    corr = apply( do.call(rbind, lapply(l["corr",], sort)), 2, median ),
    tanimoto = apply( do.call(rbind, lapply(l["tanimoto",], sort)), 2, median ),
    ATC_prefix = prefix,
    kind = "randomized correlations")
}

getPairedCorr <- function(prefix) {
  bind_rows(
    avgSampleCorr(prefix),
    dhalf %>% filter(stri_startswith_fixed(common_ATC, prefix)) %>% select(tanimoto, corr) %>% mutate(ATC_prefix = prefix, kind = "real correlations")
  )
}

dm <- bind_rows(getPairedCorr("N05A"), getPairedCorr("L01B"), getPairedCorr("C08"))

dm <- dm %>% group_by(ATC_prefix, kind) %>%
    add_count() %>%
    ungroup() %>% mutate(name = recode(ATC_prefix,
    C08 = "Calcium channel blockers",
    L01B = "Antimetabolites",
    N05A = "Antipsychotics"),
    label = paste0(name, "\n(N = " , n, ", ", ATC_prefix, ")"))

dm <- dm %>% mutate(kind_ = recode(kind,
  "randomized correlations" = "expected correlations based on 2D chemical similarity",
  "real correlations" = "actual correlations"))

dm <- dm %>% mutate(kind_ = factor(kind_)) %>% arrange(desc(ATC_prefix)) %>%
  mutate(label = fct_inorder(label))

dm %>% group_by(name) %>% do(
  wilcox.test( .[.$kind == "randomized correlations",]$corr, .[.$kind != "randomized correlations",]$corr ) %>% tidy()
)

ggplot(dm, aes(corr, label, color=kind_)) +
  geom_boxploth(outlier.size = 0.25, position = position_dodgev(0.9), notch = T, size=0.25) + theme_minimal() +
  scale_color_manual(breaks = rev(levels(dm$kind_)), values = c("black", "grey"), name="") +
  xlab("Spearman correlation between activity profiles") +
  ylab("") +
  theme(panel.grid.major.y  = element_blank(), legend.position = "bottom",
        axis.text = element_text(size = 6, color = "black"),
        axis.title = element_text(size = 7, color = "black"),
        legend.title = element_text(size = 7), legend.text = element_text(size = 6),
        legend.direction = "vertical", legend.margin = margin(-1, unit = "lines"),
        legend.key.height = unit(0.5, "lines")
  )

ggsave("figure_pdf/figED10b.pdf", width = 9, height = 4, units = "cm")



p1 <- ggplot(d1, aes(as.integer(chemical_name1), as.integer(chemical_name2), fill=corr)) + geom_raster() + scale_fill_gradient2() +
  scale_y_continuous(breaks = 1:length(compound_order), labels = compounds$chemical_name, limits = L, expand = c(0,0)) +
  scale_x_continuous(expand = c(0,0), limits = LX) +
  geom_text(data = compounds, aes(y = as.integer(prestwick_ID), label = label, color = kind), x = N+1, inherit.aes = F, size = 1, hjust = "left") +
  scale_color_manual(values = c('#e41a1c','#377eb8','#4daf4a', "#984ea3", "black")) +
  theme_minimal() +
  theme(axis.text.x = element_blank(), panel.grid = element_blank(), legend.position = "right",
        axis.text.y = element_text(size = 5), axis.title = element_blank())

p2 <- ggplot(d2, aes(as.integer(chemical_name1), as.integer(chemical_name2), fill=tanimoto)) + geom_raster() +
  scale_fill_viridis(option = "A", direction = -1) +
  theme_minimal() + scale_y_continuous(limits = L, expand = c(0,0)) + scale_x_continuous(limits = LX, expand = c(0,0)) +
  theme(axis.text.x = element_blank(), panel.grid = element_blank())

p <- p1 + annotation_custom(gtable_filter(ggplotGrob(p2), "panel"))

ggsave("figure_pdf/figED10a_tanimoto.pdf", p2, width = 8.27, height = 11.69)

ggsave("figure_pdf/figED10a_heatmap1.pdf", p, width = 8.27, height = 11.69)

ap <- compounds %>% filter(kind != "other")
ap <- ap %>% arrange(desc(ATC)) %>% mutate(chemical_name = fct_inorder(chemical_name, T))

dd <- d %>% filter(prestwick_ID1 %in% ap$prestwick_ID, prestwick_ID2 %in% ap$prestwick_ID) %>%
  arrange(chemical_name1, chemical_name2) %>%
  mutate(chemical_name1 = ordered(chemical_name1, levels = levels(ap$chemical_name)),
         chemical_name2 = ordered(chemical_name2, levels = levels(ap$chemical_name)))
d1 <- dd %>% filter(chemical_name1 < chemical_name2)
d2 <- dd %>% filter(chemical_name1 > chemical_name2)

N <- nrow(ap)
L <- c(0.5, (0.5+N))
LX <- c(0.5, (0.5+N+30))


filterATC <- function(a) {
  ifelse( stri_startswith_fixed(a, "C08"), stri_sub(a, 1, 4), stri_sub(a, 1, 5))
}

ddc <- dd %>% mutate(c1 = filterATC(ATC1), c2 = filterATC(ATC2)) %>%
  filter(chemical_name1 != chemical_name2) %>%
  group_by(c1, c2) %>% summarise(corr = median(corr), tanimoto = median(tanimoto)) %>%
  ungroup() %>% arrange(desc(c1), desc(c2))

ddc <- ddc %>% mutate(c1 = ordered(c1, levels=unique(c1)), c2 = ordered(c2, levels=unique(c1)))

dc1 <- ddc %>% filter(c1 <= c2)
dc2 <- ddc %>% filter(c1 >= c2)

dex <- tribble(
  ~xmax, ~xmin, ~ymax, ~ymin,
  "N05AF", "N05AX", "N05AA", "N05AC",
  "N05AA", "N05AC", "N05AF", "N05AX",
  "N05AA", "N05AC", "N05AA", "N05AC"
) %>% mutate_all(function(l) ordered(l, levels=levels(ddc$c1)))

ap_atc <- atc_tree %>% filter(ATC %in% dc1$c1) %>% arrange(desc(ATC))

ap_atc <- ap_atc %>% mutate(name = recode(ATC,
  C08C = "Selective calcium channel blockers",
  C08E = "Non-selective calcium channel blockers",
  .default = ATC_name))

N <- nrow(ap_atc)
L <- c(0.4, (0.5+N))
LX <- NA
# LX <- c(0.4, (0.5+N+30))

p1 <- ggplot(dc1, aes(as.integer(c1), as.integer(c2), fill=corr)) + geom_raster() +
  scale_fill_gradient2(name = "Spearman correlation\nbetween activity profiles") +
  scale_y_continuous(breaks = 1:nrow(ap_atc), labels = ap_atc$name, limits = L, expand = c(0,0)) +
  scale_x_continuous(expand = c(0,0), limits = L) +
  # geom_text(data = ap, aes(y = as.integer(chemical_name), label = label), x = N+1, inherit.aes = F, size = 2.5, hjust = "left") +
  # scale_color_manual(values = c('#e41a1c','#377eb8','#4daf4a', "#984ea3", "black")) +
  theme_minimal() +
  theme(axis.text.x = element_blank(), panel.grid = element_blank(), legend.position = "top",
        axis.text.y = element_text(size = 6, color = "black"), axis.title = element_blank(),
        legend.title = element_text(size = 7), legend.text = element_text(size = 6),
        legend.key.height = unit(0.5, "line"), #legend.key.width = unit(1, "line")
        ) +
  coord_fixed()

dtri <- dc2 %>% filter(c1 == c2) %>% mutate(x = as.integer(c1), y = as.integer(c2)) %>% rowwise() %>%
  do(tibble(g = paste(.$c1, .$c2), tanimoto = .$tanimoto, x = c(.$x-0.5, .$x+0.5, .$x+0.5), y = c(.$y-0.5, .$y-0.5, .$y+0.5)) )

p2 <- ggplot(dc2 %>% filter(c1 != c2), aes(as.integer(c1), as.integer(c2), fill=tanimoto)) + geom_raster() +
  geom_polygon(data = dtri, aes(x=x, y=y, group=g)) +
  geom_rect(data = dex, aes(xmin = as.integer(xmin)-0.5, xmax = as.integer(xmax)+0.5, ymin = as.integer(ymin)-0.5, ymax = as.integer(ymax)+0.5), fill=NA, color="black", size = 0.25, inherit.aes = F) +
  scale_fill_viridis(option = "A", direction = -1, name = "2D chemical similarity\n(Tanimoto score)") +
  theme_minimal() + scale_y_continuous(limits = L, expand = c(0,0)) + scale_x_continuous(limits = L, expand = c(0,0)) +
  theme(axis.text.x = element_blank(), panel.grid = element_blank(),
        legend.position = "bottom", legend.key.height = unit(0.5, "line"), #legend.key.width = unit(1, "line"),
        legend.title = element_text(size = 7), legend.text = element_text(size = 6)) + coord_fixed()

ggsave("figure_pdf/fig4b_tanimoto.pdf", p2, width = 15, height = 10, units = "cm")

p <- p1 + annotation_custom(gtable_filter(ggplotGrob(p2), "panel"))
p

ggsave("figure_pdf/fig4b_heatmap_no_legend.pdf", p + theme(legend.position = "none"), width = 10, height = 4.5, units = "cm")
ggsave("figure_pdf/fig4b_heatmap.pdf", p, width = 10, height = 10, units = "cm")

dc2 %>% rename(group1 = c1, group2 = c2, profile_correlation = corr) %>% arrange(group1, group2) %>% write_tsv("figure_source_data/4b.tsv")
