#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(tidyverse, stringi, here)
})

source("processing_config.R")

# output_data_path <- "/g/bork5/mkuhn/data/drugbug/output/"

species_overview <- read_tsv(paste0(output_data_path, "species_overview.tsv"), col_types = cols(
  NT_code = col_character(),
  species_name = col_character(),
  strain = col_integer(),
  specI_cluster = col_character(),
  species = col_character(),
  is_unique = col_logical(),
  species_label = col_character(),
  biosafety = col_character()
))

suppressWarnings(gram <- read_tsv("annotations/species_annotations.tsv", col_types = cols(
  .default = col_character(),
  `Risk group` = col_integer()
)) %>% select(NT_code, Gram_stain))

combined_pv <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))

extra_pv <- read_tsv(paste0(output_data_path, "extra_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))


combined_pv$veterinary <- str_detect(combined_pv$target_species, "^veterinary:")
combined_pv$human_use <- !str_detect(combined_pv$target_species, "^not a drug") & !combined_pv$veterinary

combined_hits <- combined_pv %>% group_by(target_species, prestwick_ID, human_use) %>% summarise(n_hit = sum(hit), hit = any(hit))

target_species_mapping3 <- tibble::frame_data(
  ~target_species, ~target_species_group3,
  "bacteria", "antibacterial drugs",
  "antiseptic", "antibacterial drugs",
  "fungi", "other anti-inf.",
  "metazoan parasites", "other anti-inf.",
  "protozoa", "other anti-inf.",
  "viruses", "other anti-inf.",
  "human", "human-targeted drugs"
)

combined_pv <- combined_pv %>% left_join(target_species_mapping3, by = "target_species")

abx_human_long <- combined_pv %>%
  group_by(NT_code, target_species_group3) %>% summarise(N = sum(hit)) %>% filter(!is.na(target_species_group3))

extra_abx_human_long <- extra_pv %>% left_join(target_species_mapping3, by = "target_species") %>%
  group_by(NT_code, target_species_group3) %>% summarise(N = sum(hit)) %>% filter(!is.na(target_species_group3))

abx_human <- abx_human_long %>% spread(target_species_group3, N)

abx_human <- abx_human %>% left_join(species_overview %>% select(NT_code, species_label), by = "NT_code")
abx_human <- abx_human %>% left_join(gram, by = "NT_code")

extra_abx_human <- bind_rows(abx_human_long, extra_abx_human_long) %>% spread(target_species_group3, N) %>%
  mutate(species_label = recode(NT_code,
                                NT5084 = "wildtype",
                                NT5085 = "tolC mutant",
                                NT5065 = "",
                                NT5057 = "",
                                .default = ""))

abx_human$r <- rank(abx_human$`human-targeted drugs`)

hit_counts <- combined_pv %>% filter(hit) %>% select(prestwick_ID, target_species_group3) %>% unique() %>% count(target_species_group3)
all_counts <- combined_pv %>% select(prestwick_ID, target_species_group3) %>% unique() %>% count(target_species_group3)

Nabx <- (hit_counts %>% filter(target_species_group3=="antibacterial drugs"))$n
Nhum <- (hit_counts %>% filter(target_species_group3=="human-targeted drugs"))$n
Dabx <- (all_counts %>% filter(target_species_group3=="antibacterial drugs"))$n
Dhum <- (all_counts %>% filter(target_species_group3=="human-targeted drugs"))$n

xx <- abx_human %>% mutate(abx = `antibacterial drugs`, hum = `human-targeted drugs`) %>% select(NT_code, abx, hum) %>%
  mutate(OR = hum / (Nhum-hum) / (abx / (Nabx-abx)) )

OR <- median(xx$OR)

## Odds ratio
n <- nls( hum / (Nhum-hum) ~ OR * abx / (Nabx-abx), data = xx, start = list(OR = OR)  )
OR <- coef(n)

abx2htd_OR <- function(X) {
  A <- OR * X / (Nabx-X)
  Nhum * A/(1+A)
}

X <- seq(min(abx_human$`antibacterial drugs`), Nabx-1)
Y <- abx2htd_OR(X)

dd <- tibble(`antibacterial drugs`= X, `human-targeted drugs` = Y )

dd <- dd %>% filter(`human-targeted drugs` <= max(abx_human$`human-targeted drugs`)+5)

extra_abx_human2 <- extra_abx_human %>% filter(species_label != "") %>%
  mutate(elabel =
           recode(NT_code,
                  "NT5084" = "~~italic('E. coli')~wt",
                  "NT5085" = "~~italic('E. coli')~Delta*italic('tolC')"))

real_corr <- cor(abx_human$`antibacterial drugs`, abx_human$`human-targeted drugs`, method="s")

# r <- replicate(100000, cor(abx_human$`antibacterial drugs`,sample(abx_human$`human-targeted drugs`), method="s"))
# mean(real_corr <= r)

curve_label <- tribble(~label, ~`antibacterial drugs`, ~`human-targeted drugs`,
                       sprintf("odds~ratio == %.2f", OR), Inf, Inf,
                       sprintf("r[s] == %.2f", real_corr), -Inf, Inf
)

xNT5084 <- extra_abx_human %>% filter(NT_code == "NT5084") %>% mutate(f_species = "Escherichia coli")
xNT5085 <- extra_abx_human %>% filter(NT_code == "NT5085") %>% mutate(f_species = "Escherichia coli")
xNT5002 <- extra_abx_human %>% filter(NT_code == "NT5002")
xNT5065 <- extra_abx_human %>% filter(NT_code == "NT5065") %>% mutate(f_species = "Bacteroides uniformis")
xNT5003 <- extra_abx_human %>% filter(NT_code == "NT5003")
xNT5033 <- extra_abx_human %>% filter(NT_code == "NT5033")
xNT5057 <- extra_abx_human %>% filter(NT_code == "NT5057") %>% mutate(f_species = "Bacteroides fragilis")

trig <- . %>% mutate(angle = atan2( xend-x, yend-y ),
                     line_x_end = xend - 0.8 * sin(angle),
                     line_y_end = yend - 0.8 * cos(angle),
                     arrow_x_begin = xend - sin(angle),
                     arrow_y_begin = yend - cos(angle)
)

d_arrow1 <- tibble(x = as.numeric(xNT5084[2]), xend = as.numeric(xNT5085[2]),
                   y = as.numeric(xNT5084[3]), yend = as.numeric(xNT5085[3])) %>% trig()
d_arrow2 <- tibble(x = as.numeric(xNT5002[2]), xend = as.numeric(xNT5065[2]),
                   y = as.numeric(xNT5002[3]), yend = as.numeric(xNT5065[3])) %>% trig()
d_arrow3 <- tibble(x = as.numeric(xNT5033[2]), xend = as.numeric(xNT5057[2]),
                   y = as.numeric(xNT5033[3]), yend = as.numeric(xNT5057[3])) %>% trig()

d_arrow <- bind_rows(d_arrow1, d_arrow2, d_arrow3)

abx_human <- abx_human %>% ungroup() %>%
  mutate(species = stri_replace_all_regex(species_label, "^([^ ]+ [^ ]+).*", "$1")) %>%
  mutate(f_species = fct_other(species, keep = c("Bacteroides uniformis", "Bacteroides fragilis", "Escherichia coli")))

extra_abx_human3 <- bind_rows(xNT5084, xNT5085, xNT5057, xNT5065)

species_labels <- abx_human %>% filter(r <= 1 | r > 37 | NT_code == "NT5083") %>% arrange(r) %>%
  mutate( species_label = recode(species_label, `Clostridium difficile` = "C. difficile" ) )

bind_rows(abx_human, extra_abx_human3) %>%
  select(NT_code, species_label, Gram_stain, `antibacterial drugs`, `human-targeted drugs`) %>%
  write_tsv("figure_source_data/5a.tsv")

p <- ggplot(abx_human, aes(`antibacterial drugs`, `human-targeted drugs`,
                           color = Gram_stain)) +
  geom_segment(data=d_arrow, aes(x=x, xend=line_x_end, y=y, yend=line_y_end),
               inherit.aes = F, color = "grey", linetype="solid", size = 0.25) +
  geom_segment(data=d_arrow, aes(x=arrow_x_begin, xend=line_x_end, y=arrow_y_begin, yend=line_y_end), arrow = arrow(length = unit(0.015, "npc"), type = "closed"),
               inherit.aes = F, color = "grey") +
  geom_text(data = extra_abx_human2, aes(label = elabel), color = "black", parse=T, hjust="left", vjust="top", size=2) +
  geom_text(data = species_labels,
            aes(label = species_label), color = "black", hjust = c(0, 0, 1, 1, 1), vjust = c(0.5, 0.5, 0.5, 0.5, 0.5),
            nudge_x = c(1, 0.5, -1, -1, -1), nudge_y = c(0, 0, 0, 0, 0),
            fontface = "italic", size=2) +
  geom_text(data = curve_label, aes(label=label), color = "black", parse = T, vjust = "inward", hjust = "inward", size=2) +
  geom_line(data=dd, color="black") +
  geom_point(data = extra_abx_human3, color="black", size=1, aes(shape = f_species)) +
  geom_point(size=1, aes(shape = f_species)) + coord_fixed() +
  theme_minimal() +
  scale_x_continuous(breaks = seq(60, 100, 20)) +
  scale_y_continuous(breaks = seq(0, 60, 20)) +
  scale_color_discrete(name = "Gram stain") +
  scale_shape_manual(name = "Species", values = c(15, 18, 3, 16)) +
  xlab("Number of antibacterial drugs with anticommensal activity") +
  ylab("Number of human-targeted drugs with anticommensal activity") +
  theme(legend.position = "bottom",
        axis.text = element_text(size = 6),  axis.title = element_text(size = 7),
        legend.title = element_text(size = 7),
        legend.text = element_text(size = 6),
        legend.box.margin = margin(0),
        legend.margin = margin(-0.2, unit = "line"),
        legend.key.height = unit(0.75, "line"),
        legend.direction = "vertical"
  )

p

ggsave("figure_pdf/fig5a_human_vs_abx_gram_tolC.pdf", width = 7.5, height = 11, units="cm")
