#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(tidyverse, scales, here)
})

source(here("processing_config.R"))


combined_pv <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))

pretty_target_species_mapping <- tibble::frame_data(
  ~target_species, ~target_species_,
  "bacteria", "antibiotics",
  "antiseptic", "antiseptics",
  "fungi", "antifungals",
  "metazoan parasites", "antiparasitics",
  "protozoa", "antiprotozoals",
  "viruses", "antivirals",
  "human", "human-targeted drugs",
  "veterinary: infections", "vet: anti-infectives",
  "veterinary: host animal", "vet: animal-targeted drugs",
  "veterinary: metazoan parasites", "vet: antiparasitics",
  "not a drug", "non-drugs"
)

combined_pv <- combined_pv %>% left_join(pretty_target_species_mapping, by="target_species")

compounds <- combined_pv %>% group_by(prestwick_ID, target_species_) %>% summarise( min_fdr = min(pv_comb_fdr_BH) )

compounds <- bind_rows(compounds %>% mutate(target_species_ = "all compounds"), compounds)

compound_stats <- compounds %>% group_by(target_species_) %>% arrange(min_fdr) %>% mutate(N = row_number()) %>%
  group_by(target_species_, min_fdr) %>% filter(N == max(N))

compound_stats <- compound_stats %>% ungroup() %>% mutate(bin = cut(min_fdr, c(0, 0.01, 0.1))) %>%
  group_by(target_species_, bin) %>% mutate(is_label = N == max(N))

thresholds <- compound_stats %>% filter(is_label)

p <- ggplot(compound_stats, aes(min_fdr, N)) + facet_wrap(~target_species_, scales="free_y", ncol = 3) +
  geom_linerange(data = thresholds %>% filter(bin == "(0,0.01]"), aes(x = 0.01, ymin = -Inf, ymax = N), color="darkgrey") +
  geom_linerange(data = thresholds %>% filter(bin == "(0.01,0.1]"), aes(x = 0.1, ymin = -Inf, ymax = N), color="darkgrey") +
  geom_step() +
  scale_x_log10(name="FDR threshold", breaks = 10**seq(-6, 0, 2)) +
  scale_y_continuous(breaks = pretty_breaks(n=3), expand = c(0.1, 0),
                     name = "Number of drugs with anticommensal activity") +
  expand_limits(y=0) +
  geom_point(data = thresholds) +
  geom_text(data = thresholds, aes(label = paste0(N, " ")), hjust = 1, vjust = 0) +
  theme_minimal() +
  theme(strip.background = element_rect(color = "lightgrey", fill = "lightgrey"))

compound_stats_ <- compound_stats %>% filter(target_species_ %in% c("all compounds", "human-targeted drugs"))
thresholds_ <- thresholds %>% filter(target_species_ %in% c("all compounds", "human-targeted drugs"))

p <- ggplot(compound_stats_, aes(min_fdr, N)) + facet_wrap(~target_species_, scales="free_y", ncol = 2) +
  geom_linerange(data = thresholds_ %>% filter(bin == "(0,0.01]"), aes(x = 0.01, ymin = -Inf, ymax = N), color="darkgrey") +
  geom_linerange(data = thresholds_ %>% filter(bin == "(0.01,0.1]"), aes(x = 0.1, ymin = -Inf, ymax = N), color="darkgrey") +
  geom_step() +
  scale_x_log10(name="FDR threshold", breaks = 10**seq(-6, 0, 2)) +
  scale_y_continuous(breaks = pretty_breaks(n=3), expand = c(0.1, 0),
                     name = "Number of drugs\nwith anticommensal activity") +
  expand_limits(y=0) +
  geom_point(data = thresholds_) +
  geom_text(data = thresholds_, aes(label = paste0(N, " ")), hjust = 1, vjust = 0) +
  theme_minimal() +
  theme(strip.background = element_rect(color = "lightgrey", fill = "lightgrey"))

ggsave("figure_pdf/figED5b_fdr_cutoff.pdf", p, width = 5, height = 3)

