#!/usr/bin/env Rscript

source("processing_config.R")

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(broom, stringr, readr, tibble, dplyr, ggplot2, gplots, viridis, tidyr, scales)
})

combined_pv <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))

suppressWarnings(gram <- read_tsv("annotations/species_annotations.tsv", col_types = cols(
  .default = col_character(),
  `Risk group` = col_integer()
)) %>% select(NT_code, Gram_stain))

combined_pv <- combined_pv %>% left_join(gram, by="NT_code")

# combined_pv %>% select(NT_code, Gram_stain) %>% unique() %>% write_tsv("~/Desktop/gram.tsv")
combined_pv %>% select(NT_code, Gram_stain) %>% unique() %>% group_by(Gram_stain) %>% tally()

combined_hits <- combined_pv %>% group_by(target_species, prestwick_ID, human_use) %>%
  summarise(
    n_hits_gp = sum(hit[Gram_stain == "positive"]), hit_gp = n_hits_gp > 0,
    n_hits_gn = sum(hit[Gram_stain == "negative"]), hit_gn = n_hits_gn > 0,
    n_hits = sum(hit), hit = n_hits > 0
  )


full_chemprop <- read_tsv(paste0(output_data_path, "compound_properties.tsv"), col_types = cols(
  prestwick_ID = col_character(),
  CID_flat = col_integer(),
  CID_active = col_integer(),
  smiles = col_character(),
  main_component_smiles = col_character(),
  CID_main = col_integer(),
  MolecularFormula = col_character(),
  MolecularWeight = col_double(),
  XLogP = col_double(),
  TPSA = col_double(),
  Complexity = col_integer(),
  Volume3D = col_double()
)) %>%
  mutate(Complexity_Log10 = log10(Complexity), MolecularWeight_Log10 = log10(MolecularWeight),
         TPSA_Log10 = log10(TPSA+1))

chemical_names <- read_tsv(paste0(output_data_path, "p2at.tsv")) %>% filter(!is.na(prestwick_ID)) %>% select(prestwick_ID, chemical_name) %>% unique()

all_chemprop <- full_chemprop %>%
  select(prestwick_ID, MolecularWeight_Log10, XLogP, TPSA_Log10, Complexity_Log10, Volume3D,
         MolecularWeight, TPSA, Complexity)

# all_chemprop %>% gather("property", "value", 2:ncol(all_chemprop)) %>% group_by(property) %>%
#   do(shapiro.test(.$value) %>% tidy()) %>% arrange(p.value)

# ggplot(all_chemprop %>% gather("property", "value", 2:ncol(all_chemprop)), aes(value)) + facet_wrap(~property, scale="free") +
#   geom_histogram(bins = 20)

chemprop <- all_chemprop %>% select(prestwick_ID, MolecularWeight_Log10, XLogP, TPSA_Log10, Complexity_Log10, Volume3D)

chemprop <- chemprop %>% gather("property", "value", 2:ncol(chemprop))

chemprop <- chemprop %>% mutate(property = recode(property,
    "MolecularWeight_Log10" = "mol. weight (log10)",
    "Complexity_Log10" = "complexity (log10)",
    "TPSA_Log10" = "TPSA (log10)",
    "Volume3D" = "volume"
  ))

chemprop <- chemprop %>% left_join(chemical_names, by = "prestwick_ID")

chemprop_hits <- combined_hits %>% left_join(chemprop, by="prestwick_ID") %>% filter(human_use)

chemprop_hits <- chemprop_hits %>% filter(!is.na(value))
chemprop_hits$target_species_b <- ifelse(chemprop_hits$target_species == "bacteria", "antibacterial drugs",
  ifelse(chemprop_hits$target_species == "human", "human-targeted drugs", "other anti-infectives"))

chemprop_hits_g <- chemprop_hits %>% select(-hit_gp, -hit_gn, -hit) %>%
  gather("gram", "hit_count", which(colnames(.) %in% c("n_hits_gp", "n_hits_gn", "n_hits")))

chemprop_hits_g <- chemprop_hits_g %>% left_join(combined_hits %>% ungroup() %>% select(prestwick_ID, n_hits), by="prestwick_ID")

chemprop_hits_g$hit <- chemprop_hits_g$hit_count > 0

chemprop_hits_gp <- chemprop_hits_g %>% filter(gram != "n_hits", target_species_b != "other anti-infectives") %>%
  group_by(gram) %>% mutate(hit_fraction = hit_count/max(hit_count))

chem_hits_anova <-
  chemprop_hits_gp %>% group_by(target_species_b, property) %>% filter(n_hits > 0) %>%
  do(car::Anova(lm(value ~ hit_fraction * gram, data=.)) %>% tidy())


ct <- chem_hits_anova %>% arrange(p.value) %>% filter(p.value < 0.05) %>%
  mutate(kind = recode(term,
                  "hit_fraction" = "slope",
                  "hit_fraction:gram" = "slope difference",
                  "gram" = "shift"),
  label = paste0(kind, ": p = ", sprintf("%.2g", p.value))) %>%
  group_by(target_species_b, property) %>% arrange(label) %>%
  summarise(label = paste(label, collapse="\n"))

ggplot(chemprop_hits_gp %>% filter(n_hits > 0), aes(hit_fraction, value, color=gram)) +
  geom_point(size=1, alpha=0.3, shape=17) + facet_grid(property~target_species_b, scales="free") +
  geom_smooth(method="lm", show.legend = F) +
  xlab("Fraction of affected strains per drug") + ylab("Value of chemical property") +
  geom_text(data = ct, aes(label = label), x = 0, y = Inf, color = "black", hjust = "inward", vjust = 1.2, inherit.aes = F) +
  scale_color_discrete(labels = c("negative", "positive"), name = "Gram stain of affected species") +
  scale_y_continuous(breaks = trans_breaks(identity, identity, n = 3)) +
  guides(colour = guide_legend(override.aes = list(alpha=1))) +
  theme_minimal() + theme(strip.background = element_rect(fill="lightgrey", color="white"), legend.position = "bottom",
                          panel.grid.minor = element_blank())

ggsave("figure_pdf/figED11_chemprop.pdf", width = 6, height = 8)
