#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(tidyverse, stringi, ggstance, here, stringr, forcats, ggforce, broom, viridis, reshape2)
})

source(here("processing_config.R"))

combined_pv_full <- read_tsv(paste0(output_data_path, "combined_pv.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  pv_comb_fdr_BH = col_double(),
  AUC = col_double(),
  hit = col_logical(),
  target_species = col_character()
))

chemical_names <- read_tsv(paste0(output_data_path, "p2at.tsv")) %>% filter(!is.na(prestwick_ID)) %>% select(prestwick_ID, chemical_name) %>% unique()
combined_pv_full <- combined_pv_full %>% inner_join(chemical_names, by="prestwick_ID")

atc_tree <- read_tsv(paste0(output_data_path, "atc_tree"), col_types = cols(), col_names = c("ATC", "name"))

prestwick_atc <- read_tsv(paste0(output_data_path, "prestwick_atc.tsv"), col_types = cols())
prestwick_atc3 <- prestwick_atc %>% mutate(ATC3 = stri_sub(ATC, 1, 4)) %>% select(prestwick_ID, ATC3) %>% unique()
prestwick_atc4 <- prestwick_atc %>% mutate(ATC4 = stri_sub(ATC, 1, 5)) %>% select(prestwick_ID, ATC4) %>% unique()

species_names <- read_tsv(paste0(output_data_path, "species_overview.tsv"))
combined_pv_full <- combined_pv_full %>% inner_join(species_names)

AAP <- c("Prestw-350", "Prestw-1323", "Prestw-1229", "Prestw-1455", "Prestw-1029", "Prestw-1231", "Prestw-1188")

combined_pv <- combined_pv_full %>% filter(target_species == "human") %>% inner_join(prestwick_atc4) %>%
  mutate(is_antipsychotic = stri_startswith_fixed(ATC4, "N05A"),
    is_aap = prestwick_ID %in% AAP) %>% select(-ATC4) %>%
  group_by(NT_code, species_label, prestwick_ID, chemical_name, AUC, pv_comb, pv_comb_fdr_BH, hit) %>%
  summarise(is_antipsychotic = any(is_antipsychotic), is_aap = any(is_aap)) %>%
  ungroup() %>%
  mutate(kind = ifelse(is_aap, "Atypical antipsychotics", ifelse(is_antipsychotic, "Other antipsychotics", "Other human-target drugs")))

akk <- combined_pv %>% filter(NT_code == "NT5021")

akk %>% group_by(is_antipsychotic) %>% summarise(f_hit = sum(hit)/n())
akk %>% group_by(is_aap) %>% summarise(f_hit = sum(hit)/n())

combined_pv %>% group_by(NT_code, species_label) %>% do({
  m <- .$is_aap
  wilcox.test(.[m,]$pv_comb_fdr_BH, .[!m,]$pv_comb_fdr_BH, alternative = "l") %>% tidy()
}) %>% arrange(p.value)

combined_pv %>% group_by(kind) %>% do({
  m <- .$NT_code == "NT5021"
  wilcox.test(.[m,]$pv_comb_fdr_BH, .[!m,]$pv_comb_fdr_BH, alternative = "l") %>% tidy()
})

combined_pv %>% group_by(kind) %>% do({
  m <- .$NT_code == "NT5021"
  wilcox.test(.[m,]$pv_comb_fdr_BH, .[!m,]$pv_comb_fdr_BH, alternative = "t") %>% tidy()
})

ggplot(combined_pv, aes(pv_comb_fdr_BH, color = NT_code == "NT5021")) + geom_density() + facet_wrap(~kind)


combined_pv_h <- bind_rows(
  combined_pv %>% mutate(kind = "All human-targeted drugs"),
  combined_pv %>% filter(is_antipsychotic) %>% mutate(kind = "All antipsychotics"),
  combined_pv %>% filter(is_aap)
) %>% mutate(species = ifelse(NT_code == "NT5021", species_label, "Other species"))



ggplot(combined_pv_h %>% filter(!(kind == "Atypical antipsychotics" & species == "Akkermansia muciniphila")), aes(pv_comb, color = species)) +
  geom_density() +
  geom_point(data = combined_pv_h %>% filter(kind == "Atypical antipsychotics", species == "Akkermansia muciniphila"), aes(y=0)) +
  facet_wrap(~kind, ncol=1, scales="free")

combined_pv_h <- combined_pv_h %>% mutate(kind = factor(kind, levels = c("Atypical antipsychotics", "All antipsychotics", "All human-targeted drugs")))
combined_pv_h <- combined_pv_h %>% mutate(species = fct_relevel(species, "Other species"))

ggplot(combined_pv_h %>% filter(!(kind == "Atypical antipsychotics" & species == "Akkermansia muciniphila")), aes(pv_comb, species)) +
  geom_boxploth() +
  geom_point(data = combined_pv_h %>% filter(kind == "Atypical antipsychotics", species == "Akkermansia muciniphila")) +
  facet_wrap(~kind, ncol=1)

combined_pv_h <- combined_pv_h %>% group_by(kind) %>% mutate(N_drug = length(unique(prestwick_ID)),
                                            kind_ = paste0(kind, " (N = ", N_drug, ")")) %>% ungroup() %>%
  mutate(kind_ = fct_reorder(kind_, N_drug))

pvs <- combined_pv_h %>% group_by(kind_) %>% do({
  m <- .$NT_code == "NT5021"
  wilcox.test(.[m,]$pv_comb, .[!m,]$pv_comb, alternative = "t") %>% tidy()
})

pvs <- pvs %>% mutate(label = sprintf(" p = %.1g", p.value))

combined_pv_h <- combined_pv_h %>% group_by(kind_, species) %>% mutate(N = n())

ggplot(combined_pv_h, aes(species, pv_comb)) +
  geom_violin(aes(color = N > 10), draw_quantiles = 0.5, show.legend = F) +
  geom_point(data = combined_pv_h %>% filter(N <= 10), size = 0.5) +
  scale_color_manual(values = c("grey", "black")) +
  coord_flip() +
  geom_text(data = pvs, aes(x = 1.5, y = 1, label = label), size = 3, hjust = "outward") +
  facet_wrap(~kind_, ncol=1) + xlab("") + ylab("p-value of anticommensal activity") +
  scale_y_continuous(breaks = seq(0, 1, 0.25), limits = c(-0.01, 1.32), expand = c(0,0)) +
  theme_minimal() +
  theme(axis.text.y = element_text(hjust = 0, size = 6), strip.text = element_text(size = 8),
        strip.background = element_rect(color = "white", fill = "lightgrey"),
        panel.grid.major.y = element_blank(), panel.grid.minor = element_blank(),
        axis.text.x = element_text(size = 6),  axis.title.x = element_text(size = 7))

ggsave("figure_pdf/figED7c_Akkermansia_AAP.pdf", width = 10, height = 6, units = "cm")

combined_pv %>% group_by(kind) %>% do({
  m <- .$NT_code == "NT5021"
  wilcox.test(.[m,]$pv_comb_fdr_BH, .[!m,]$pv_comb_fdr_BH, alternative = "l") %>% tidy()
})

combined_pv %>% group_by(kind) %>% do({
  m <- .$NT_code == "NT5021"
  wilcox.test(.[m,]$pv_comb_fdr_BH, .[!m,]$pv_comb_fdr_BH, alternative = "g") %>% tidy()
})

combined_pv %>% group_by(kind) %>% do({
  m <- .$NT_code == "NT5021"
  wilcox.test(.[m,]$pv_comb, .[!m,]$pv_comb, alternative = "l") %>% tidy()
})

combined_pv %>% group_by(kind) %>% do({
  m <- .$NT_code == "NT5021"
  wilcox.test(.[m,]$pv_comb, .[!m,]$pv_comb) %>% tidy()
})


ranks <- combined_pv %>% group_by(kind, NT_code, species_label) %>% summarise(pv_comb = median(pv_comb)) %>%
  group_by(kind) %>% mutate(rank = rank(pv_comb)) %>% arrange(rank)

ranks %>% group_by(NT_code) %>% mutate(pv_comb_AAP = pv_comb[ kind == "Atypical antipsychotics"]) %>% ungroup() %>%
  mutate(species_label = fct_reorder(species_label, pv_comb_AAP)) %>% ggplot(aes(kind, species_label, fill = pv_comb)) + geom_raster() + scale_fill_viridis(direction = -1)

combined_pv %>% filter(is_aap) %>% group_by(kind, NT_code, species_label) %>% tally(hit) %>% ungroup() %>% count(n)

