#!/usr/bin/env Rscript

suppressPackageStartupMessages({
  if (!require("pacman")) install.packages("pacman")
  pacman::p_load(broom, metap, stringr, readr, tibble, dplyr, ggplot2, tidyr, here)
})

source("processing_config.R")
source("distributions.R")

# strains that are not part of the initial screen and that should not change the result
extra_strains <- c("NT5084", "NT5085", "NT5065", "NT5057")

# aucs_full <- read_tsv(paste0(output_data_path, "aucs_new_replicates.tsv"), col_types = cols(
aucs_full <- read_tsv(paste0(output_data_path, "aucs.tsv"), col_types = cols(
  NT_code = col_character(),
  prestwick_ID = col_character(),
  plate = col_character(),
  row = col_character(),
  col = col_integer(),
  normAUC = col_double(),
  control = col_character(),
  plate_format = col_character(),
  replicate_id = col_character()
))

aucs_full$p <- ifelse(aucs_full$prestwick_ID == "CONTROL", 1:nrow(aucs_full), aucs_full$prestwick_ID)
aucs_full <- aucs_full %>% group_by(NT_code, prestwick_ID, plate, replicate_id, p, prestwick_ID, control, plate_format, replicate_id) %>%
  summarise(normAUC = mean(normAUC), row = row[1], col = col[1]) %>% ungroup() %>%  select(-p)

message("Computing p-values by plate")
aucs_pv_by_plate <- aucs_full %>% group_by(NT_code, replicate_id, plate) %>% do({
  AUC_to_pValue_by_plate(.)
})

message("Computing p-values by replicate")
aucs_pv_by_replicate <- aucs_full %>% group_by(NT_code, replicate_id) %>% do({
  AUC_to_pValue_by_plate(.)
})

aucs_pv <- aucs_pv_by_plate %>% full_join(
  aucs_pv_by_replicate %>% select(-normAUC, -control, -plate_format),
  by = c("NT_code", "prestwick_ID", "plate", "row", "col", "replicate_id")
)

aucs_pv$pv <- pmax(aucs_pv$pv.x, aucs_pv$pv.y)

aucs_pv <- aucs_pv %>% select(-ends_with(".x"), -ends_with(".y"))

write_tsv(aucs_pv, paste0(output_data_path, "aucs_pv.tsv"))

aucs_pv_trmt <- aucs_pv %>% filter(control != "CONTROL")
write_tsv(aucs_pv_trmt, paste0(output_data_path, "aucs_pv_trmt.tsv"))

message("Combining p-values")

combined_pv <- aucs_pv_trmt %>% group_by(NT_code, prestwick_ID) %>% filter(n() > 1) %>%
  summarise(pv_comb = sumlog(pmax(1e-3, pv))[["p"]],
            AUC = mean(normAUC), AUCsd = sd(normAUC), fine = sum(plate_format == 96) > 1,
            n_repl = n())

## Remove extraneous species (E coli tolC mutant/WT that were added later,
## but are not part of the actual screen)
## before calculation of FDR
pv <- (combined_pv %>% filter(! NT_code %in% extra_strains))$pv_comb
apv <- p.adjust(pv, method = "BH")

combined_pv$pv_comb_fdr_BH <- approx(pv, apv, combined_pv$pv_comb, rule = 2)$y


p2a1 <- read_tsv(paste0(output_data_path, "p2a1.tsv"), col_types = cols(
  prestwick_ID = col_character(),
  CID_flat = col_character(),
  target_species = col_character(),
  veterinary = col_logical(),
  human_use = col_logical()
))

N_species <- length(unique(combined_pv$NT_code))
combined_pv <- combined_pv %>% group_by(prestwick_ID) %>% filter(n() >= N_species/2)

combined_pv <- combined_pv %>% left_join(p2a1, by="prestwick_ID")
combined_pv$target_species <- ifelse(is.na(combined_pv$target_species),
                                     "not annotated",
                                     combined_pv$target_species)

combined_pv$hit <- combined_pv$pv_comb_fdr_BH < fdr_threshold

extra_pv <- combined_pv %>% filter(NT_code %in% extra_strains)
write_tsv(extra_pv, paste0(output_data_path, "extra_pv.tsv"))

combined_pv <- combined_pv %>% filter(!NT_code %in% extra_strains)

write_tsv(combined_pv, paste0(output_data_path, "combined_pv.tsv"))

combined_hits <- combined_pv %>% group_by(target_species, prestwick_ID, human_use) %>%
  summarise(n_hit = sum(hit), hit = n_hit > 0)

write_tsv(combined_hits, paste0(output_data_path, "combined_hits.tsv"))


